package foh.rpg;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import foh.rpg.area.factory.AreaFactory;
import foh.rpg.character.factory.ActorFactory;
import foh.rpg.gui.Gui;
import foh.rpg.gui.MenuItem;
import foh.rpg.gui.SoundSystem;
import foh.rpg.model.Model;

/**
 * @author Zweistein
 */
public class Main {

	public static void main(String[] args) {
		final Model model = new Model();

		Gui.createAndShowGUI();
		
		SoundSystem.getInstance().menuMusic();

		Gui.getConsole().printTextLn("Welcome To Fate of Heroes");
		Gui.getConsole().printTextAt(getVersionString(), 1, Gui.getConsole().getScreenResolutionY());
		Gui.getConsole().moveCursorTo(5, 3);

		List<MenuItem> menu = new ArrayList<MenuItem>();
		
		MenuItem newGame = new MenuItem(null, 'N', "New Game");
		menu.add(newGame);
		MenuItem quit = new MenuItem(null, 'Q', "Quit");
		menu.add(quit);
		
		MenuItem result = Gui.getInlineMenuItem(menu, quit, Gui.getConsole().getCursorX(), Gui.getConsole().getCursorY());
		
		if (result == newGame) {
			// NEW GAME

			model.setPc(ActorFactory.createCharacter());
			
			
			Gui.getConsole().printTextLn("");
			Gui.getConsole().printTextLn("Enter Character Name:");
			
			String name = Gui.getStringKey();
			
			model.getPc().setName(name);
			
			model.setArea(AreaFactory.getInstance().getArea(model, "lobby"));
			model.pcEnterArea(10, 10);

			SoundSystem.getInstance().exploreMusic();
			
			Thread enviromentThread = new Thread( new Runnable () {

				@Override
				public void run() {
					while (true) {
						
						model.fireMonsterEvents();
						
						model.fireEnviromentEvents();
						
						try {
							Thread.sleep(500);
						} catch (InterruptedException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
						
					}
				}
				
			});
			
			enviromentThread.start();

			while (true) {

				// now, we print messages and redraw

				Gui.refreshModelDisplay(model);
				Gui.displayModelMessages(model);

				if (Gui.isInput()) {
					int key = Gui.getNextKey().getKeyCode();
					model.fireCommandEvent(key);
				}
				
				try {
					Thread.sleep(80);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				
			}

		} else if (result == quit) {
			
			System.exit(0);

		} else {
			
			Gui.getConsole().printTextLn("WTF?");
			
		}

	}
	
	private static String getVersionString() {
		Properties properties = new Properties();
		try {
			properties.load(Main.class.getClassLoader().getResource("version.properties").openStream());
			return "Build #" + properties.getProperty("buildNum") + " from " + properties.getProperty("buildDate");
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		return "";
	}
	
}
