package foh.rpg.area;

import java.util.ArrayList;
import java.util.List;

import foh.rpg.character.Actor;
import foh.rpg.model.Model;

public class Area {
	
	private Cell[][] tile = null;
	private int width = 0;
	private int height = 0;
	private Model model;

	/**
	 * @return Returns the tile.
	 */
	public Cell[][] getTile() {
		return this.tile;
	}

	public void setLOSmap (int[][] map) {
		for (int i = 0; i < map.length; i++) {
			for (int j = 0; j < map[i].length; j++) {
				if (map[i][j] == 1) {
					this.tile[i][j].setKnown(true);
					this.tile[i][j].setVisible(true);
				} else {
					this.tile[i][j].setVisible(false);
				}
			}
			
		}
	}
	
	public void setTile(int x, int y, Cell cell) {
		this.tile[x][y] = cell;
		cell.setArea(this);
		cell.getPosition().setX(x);
		cell.getPosition().setY(y);
	}
	
	public Area(Cell defaulTile, int width, int height) {
		super();

		this.tile = new Cell[width][height];
		
		for (int x = 0; x < width; x++) {
			for (int y = 0; y < height; y++) {
				setTile(x, y, defaulTile.getClone());;
			}
			
		}
		
		this.width = width;
		this.height = height;

		
	}

	/**
	 * @return Returns the height.
	 */
	public int getHeight() {
		return this.height;
	}

	/**
	 * @return Returns the width.
	 */
	public int getWidth() {
		return this.width;
	}

	public Cell getActorTile(Actor actor) {
		for (int x = 0; x < this.getWidth(); x++) {
			for (int y = 0; y < this.getHeight(); y++) {
				if (this.tile[x][y].getActor() == actor) {
					return this.tile[x][y];
				}
				
			}
		}
		return null;
	}
	
	public List<Cell> getAdjancedTilePositionList(int x, int y) {
		List<Cell> cells = new ArrayList<Cell>();

		int tmpX = 0;
		int tmpY = 0;
		
		// NW N NE
		// W  .  E
		// SW S SE
		
		// NW
		tmpX = x-1;
		tmpY = y-1;
		if (AreaTools.checkAreaBounds(tmpX, tmpY, this) == true) {
			cells.add(this.tile[tmpX][tmpY]);
		}
		
		// N
		
		tmpX = x;
		tmpY = y-1;
		if (AreaTools.checkAreaBounds(tmpX, tmpY, this) == true) {
			cells.add(this.tile[tmpX][tmpY]);
		}
		
		// NE
		
		tmpX = x+1;
		tmpY = y-1;
		if (AreaTools.checkAreaBounds(tmpX, tmpY, this) == true) {
			cells.add(this.tile[tmpX][tmpY]);
		}
		
		
		
		// SW
		tmpX = x-1;
		tmpY = y+1;
		if (AreaTools.checkAreaBounds(tmpX, tmpY, this) == true) {
			cells.add(this.tile[tmpX][tmpY]);
		}
		
		// S
		
		tmpX = x;
		tmpY = y+1;
		if (AreaTools.checkAreaBounds(tmpX, tmpY, this) == true) {
			cells.add(this.tile[tmpX][tmpY]);
		}
		
		// SE
		
		tmpX = x+1;
		tmpY = y+1;
		if (AreaTools.checkAreaBounds(tmpX, tmpY, this) == true) {
			cells.add(this.tile[tmpX][tmpY]);
		}
		
		// W
		tmpX = x-1;
		tmpY = y;
		if (AreaTools.checkAreaBounds(tmpX, tmpY, this) == true) {
			cells.add(this.tile[tmpX][tmpY]);
		}
		
		// E
		
		tmpX = x+1;
		tmpY = y;
		if (AreaTools.checkAreaBounds(tmpX, tmpY, this) == true) {
			cells.add(this.tile[tmpX][tmpY]);
		}
		
		
		return cells;
		
	}

	public void setModel(Model model) {

		this.model = model;
		
		for (int x = 0; x < this.getWidth(); x++) {
			for (int y = 0; y < this.getHeight(); y++) {
				if (this.tile[x][y].getActor() != null) {
					this.tile[x][y].getActor().setCurrentModel(model);
				}
			}
		}

	}

	public Cell getTileByPortalKey(String targetPortal) {
		for (int x = 0; x < this.getWidth(); x++) {
			for (int y = 0; y < this.getHeight(); y++) {
				if (this.tile[x][y].getPortal() != null  && targetPortal.equals(this.tile[x][y].getPortal().getThisPortalKey())) {
					return this.tile[x][y];
				}
			}
		}
		return null;
	}

	public Model getModel() {
		return this.model;
	}
	
}
