package foh.rpg.area;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import foh.rpg.character.position.Position;
import foh.rpg.character.position.PositionDistanceComparator;
public class AreaTools {

	public static void fillRectangle (Area area, Cell tile, int x1, int y1, int x2, int y2) {
		
		for (int x = x1; x <= x2; x++) {
			for (int y = y1; y <= y2; y++) {
				area.setTile(x, y, tile.getClone());
			}
		}
		
	}

	public static void drawTileLine(Area area, Cell tile, int x0, int y0, int x1, int y1) {
		// bresenham http://www.cs.unc.edu/~mcmillan/comp136/Lecture6/Lines.html
		
        int dx = x1 - x0;
        int dy = y1 - y0;
        
        area.getTile()[x0][y0] = tile.getClone();

        if (Math.abs(dx) > Math.abs(dy)) {          // slope < 1
            float m = (float) dy / (float) dx;      // compute slope
            float b = y0 - m*x0;
            dx = (dx < 0) ? -1 : 1;
            while (x0 != x1) {
                x0 += dx;
                area.setTile(x0, Math.round(m*x0 + b), tile.getClone());
            }
        } else
        if (dy != 0) {                              // slope >= 1
            float m = (float) dx / (float) dy;      // compute slope
            float b = x0 - m*y0;
            dy = (dy < 0) ? -1 : 1;
            while (y0 != y1) {
                y0 += dy;
                area.setTile(Math.round(m*y0 + b), y0, tile.getClone());
            }
        }
		
		
	}
	
	
	public static int[][] checkLOS(Position actorPosition, Area area,
			int range) {

		int[][] map = new int[area.getWidth()][area.getHeight()];

		for (int xR = -range; xR <= range; xR++) {
			for (int yR = -range; yR <= range; yR++) {
				int txR = actorPosition.getX() + xR;
				int tyR = actorPosition.getY() + yR;
				if (AreaTools.checkAreaBounds(txR, tyR, area)) {
					if (xR * xR + yR * yR <= range * range) {

						// we cast ray to "see"

						double vecSize = Math.sqrt(xR * xR + yR * yR);

						double unifiedX = xR / vecSize;
						double unifiedY = yR / vecSize;

						boolean rayCheck = false;

						for (double i = 0; i <= vecSize; i += 0.1) {
							int stepX = (int) Math.round(unifiedX * i)
									+ actorPosition.getX();
							int stepY = (int) Math.round(unifiedY * i)
									+ actorPosition.getY();

							if (AreaTools.checkAreaBounds(stepX, stepY, area)) {

								if (!area.getTile()[stepX][stepY]
										.isSeeThrough()) {
									rayCheck = true;

									map[stepX][stepY] = 1;
									// area.getTile()[stepX][stepY].setKnown(true);
									break;

								}

							}
						}

						if (!rayCheck) {
							map[txR][tyR] = 1;
							// area.getTile()[txR][tyR].setKnown(true);
						}
					}
				}
			}
		}

		return map;

	}
	
	public static void createRayMatrix (int width, int height) {
		
		// prepare data
		int[][] matrix = new int[width][height];
		for (int x = 0; x < width; x++) {
			for (int y = 0; y < height; y++) {
				matrix[x][y] = 0;
			}
		}
		
		// we will now build raw pathstuff
		
		int posX = 0; int posY = 0;
		
		do {
			if ( posX < (width-height)) {
				posX++;
			} else {
				posX++;
				posY++;
			}
			matrix[posX][posY] = 1;
		} while ((posX < width-1) && (posY < height-1));
		
		posX = 0;
		posY = 0;
		
		do {
			if ( posX > height) {
				posX++;
			} else {
				posX++;
				posY++;
			}
			matrix[posX][posY] = 1;
		} while ((posX < width-1) && (posY < height-1));
		
	}
	
	public static boolean checkAreaBounds(int x, int y, Area area) {
		return (x >= 0 && y >= 0 && x < area.getWidth() && y < area.getHeight());
	}

	public static Position getNextPathElement(Area area, int[][] los,
			int x, int y, int targetX, int targetY) {
		
		// first, we superimpose los on area map to get fields that can be moved through;
		
		for (int iX = 0; iX < los.length; iX++) {
			for (int iY = 0; iY < los[iX].length; iY++) {
				if (los[iX][iY] == 1) {
					if (!area.getTile()[iX][iY].isMoveThrough()) {
						los[iX][iY] = 0;
					} else if (area.getTile()[iX][iY].getActor() != null) {
						los[iX][iY] = 0;
					}
				}
			}
		}
		
		// now we have los map of where actor can move along with target
		
		los[x][y] = 1; // our origin point can be moved through
		los[targetX][targetY] = -1; // mark target
		int timeout = 0;

		while (los[x][y] == 1 && timeout < 100 ) {
			
			timeout++;
			
			int[][] los2 = new int[los.length][los[0].length];
			
			for (int iX = 0; iX < los.length; iX++) {
				for (int iY = 0; iY < los[iX].length; iY++) {
					
					if (los[iX][iY] == 1) {
						
						int minimuNearby = 0;
						
						List<Cell> adjacent = area.getAdjancedTilePositionList(iX, iY);
						for (Cell cell : adjacent) {
							Position position = cell.getPosition();
							if (los[position.getX()][position.getY()] < minimuNearby) {
								minimuNearby = los[position.getX()][position.getY()];
							}
						}
						
						if (minimuNearby < 0) {
							los2[iX][iY] = minimuNearby-1;
						}
						
					}
					
				}
			}
			
			for (int iX = 0; iX < los.length; iX++) {
				for (int iY = 0; iY < los[iX].length; iY++) {
					if (los2[iX][iY] != 0) {
						los[iX][iY] = los2[iX][iY];
					}
				}
			}
		}
		
		int maximumNearby = los[x][y];
		Position result = new Position(x, y);
		
		List<Cell> adjacent = area.getAdjancedTilePositionList(x, y);
		
		for (Cell cell : adjacent) {
			Position position = cell.getPosition();
			if (los[position.getX()][position.getY()] > maximumNearby && los[position.getX()][position.getY()] < 0) {
				maximumNearby = los[position.getX()][position.getY()];
			}
		}
		
		List<Position> acceptableTargets = new ArrayList<Position>();
		for (Cell cell : adjacent) {
			Position position = cell.getPosition();
			if (maximumNearby == los[position.getX()][position.getY()]) {
				acceptableTargets.add(position);
			}
		}
		
		Collections.shuffle(acceptableTargets);
		
		if (acceptableTargets.size() > 0) {
			result = acceptableTargets.get(0);
		}
		
		return result;
	}

	public static List<Position> sortByDistance(List<Position> positions, Position pivot) {
		Collections.sort(positions, new PositionDistanceComparator(pivot));
		return positions;
	}
	
}
