package foh.rpg.area;

import java.util.ArrayList;
import java.util.List;

import foh.rpg.character.Actor;
import foh.rpg.character.Targetable;
import foh.rpg.character.effect.EffectStack;
import foh.rpg.character.position.Position;
import foh.rpg.gui.Representation;
import foh.rpg.item.Item;

public class Cell implements Targetable {
	
	private Representation graphics = null;
	private boolean known = true;
	private boolean seeThrough = true;
	private boolean moveThrough = true;
	private List<Item> items = null;
	private String description = null;
	private boolean visible = false;
	private EffectStack effectStack;
	private SpawnPoint spawnPoint;

	public SpawnPoint getSpawnPoint() {
		return this.spawnPoint;
	}

	private Position position = new Position();
	private Area area;
	
	private AreaPortal portal;
	
	private Actor actor = null;
	
	public EffectStack getEffectStack() {
		return this.effectStack;
	}
	
	public void setEffectStack(EffectStack effectStack) {
		this.effectStack = effectStack;
	}

	public Area getArea() {
		return this.area;
	}

	public void setArea(Area area) {
		this.area = area;
	}

	public Cell(Representation graphics, boolean seeThrough, boolean moveThrough, String description) {
		super();
		this.graphics = graphics;
		this.seeThrough = seeThrough;
		this.moveThrough = moveThrough;
		this.description = description;

		this.known = false;
		this.items =  new ArrayList<Item>();
		this.effectStack = new EffectStack();
		this.spawnPoint = new SpawnPoint();
		this.spawnPoint.setCell(this);
		this.actor = null;
	}

	public Cell() {
		this.known = false;
		this.items =  new ArrayList<Item>();
		this.effectStack = new EffectStack();
		this.spawnPoint = new SpawnPoint();
		this.spawnPoint.setCell(this);
		this.actor = null;
	}

	public Position getPosition() {
		return this.position;
	}

	/**
	 * @return Returns the description.
	 */
	public String getDescription() {
		return this.description;
	}

	/**
	 * @param description The description to set.
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * @return Returns the graphics.
	 */
	public Representation getGraphics() {
		return this.graphics;
	}

	/**
	 * @param graphics The graphics to set.
	 */
	public void setGraphics(Representation graphics) {
		this.graphics = graphics;
	}

	/**
	 * @return Returns the items.
	 */
	public List<Item> getItems() {
		return this.items;
	}

	/**
	 * @param items The items to set.
	 */
	public void setItems(List<Item> items) {
		this.items = items;
	}

	/**
	 * @return Returns the known.
	 */
	public boolean isKnown() {
		return this.known;
	}

	/**
	 * @param known The known to set.
	 */
	public void setKnown(boolean known) {
		this.known = known;
	}

	/**
	 * @return Returns the moveThrough.
	 */
	public boolean isMoveThrough() {
		return this.moveThrough;
	}

	/**
	 * @param moveThrough The moveThrough to set.
	 */
	public void setMoveThrough(boolean moveThrough) {
		this.moveThrough = moveThrough;
	}

	/**
	 * @return Returns the seeThrough.
	 */
	public boolean isSeeThrough() {
		return this.seeThrough;
	}

	/**
	 * @param seeThrough The seeThrough to set.
	 */
	public void setSeeThrough(boolean seeThrough) {
		this.seeThrough = seeThrough;
	}

	public String toString() {
		return "'" + this.getGraphics() + "' " + this.description;
	}

	/**
	 * @return Returns the actor.
	 */
	public Actor getActor() {
		return this.actor;
	}

	/**
	 * @param actor The actor to set.
	 */
	public void setActor(Actor actor) {
		this.actor = actor;
		if (actor != null) {
			actor.setCurrentArea(this.getArea());
		}
		this.effectStack.setTarget(actor);
	}

	public Cell getClone() {
		return new Cell(this.getGraphics(), this.isSeeThrough(), this.isMoveThrough(), this.getDescription());
	}

	public void setVisible(boolean visible) {
		this.visible = visible;
	}

	public boolean isVisible() {
		return this.visible;
	}
	
	public List<Cell> getAdjacentTiles() {
		return this.getArea().getAdjancedTilePositionList(this.getPosition().getX(), this.getPosition().getY());
	}

	public AreaPortal getPortal() {
		return this.portal;
	}

	public void setPortal(AreaPortal portal) {
		this.portal = portal;
	}

	public void tick() {
		this.effectStack.tick();
		this.spawnPoint.tick();
	}
	
}
