package foh.rpg.area.factory;

import java.util.HashMap;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import foh.rpg.area.Area;
import foh.rpg.area.AreaPortal;
import foh.rpg.area.Cell;
import foh.rpg.area.factory.painter.AreaPainter;
import foh.rpg.area.factory.painter.CellFill;
import foh.rpg.area.factory.painter.CellLine;
import foh.rpg.area.factory.painter.CellRandomize;
import foh.rpg.area.factory.painter.PaintedAreaPrototype;
import foh.rpg.area.factory.randomizer.ActorSpawn;
import foh.rpg.area.factory.randomizer.MatrixPortalSegment;
import foh.rpg.area.factory.randomizer.RandomAreaPrototype;
import foh.rpg.area.factory.randomizer.SegmentConnector;
import foh.rpg.gui.Representation;
import foh.rpg.model.Model;
import foh.rpg.util.FactoryFromXML;

public class AreaFactory extends FactoryFromXML {

	private static AreaFactory instance = null;

	private Map<String, Area> areas = null;
	private Map<String, AreaPrototype> prototypes = null;

	private AreaFactory() {

		this.setAreas(new HashMap<String, Area>());
		this.setPrototypes(new HashMap<String, AreaPrototype>());

		Document doc = this.parseDocument("world.xml");

		// TODO: each prototype should know to to parse itself...
		parsePaintedAreas(doc.getElementsByTagName("area"));

		parseRandomAreas(doc.getElementsByTagName("randomArea"));

	}
	
	private void parseRandomAreas(NodeList randomAreaNodes) {

		for (int i = 0; i < randomAreaNodes.getLength(); i++) {
			Node randomAreaNode = randomAreaNodes.item(i);
			
			RandomAreaPrototype randomAreaPrototype = new RandomAreaPrototype();
			String prototypeKey = randomAreaNode.getAttributes().getNamedItem("id").getNodeValue();
			this.getPrototypes().put(prototypeKey, randomAreaPrototype);
			
			randomAreaPrototype.setSegmentsX(Integer.parseInt(randomAreaNode
					.getAttributes().getNamedItem("segmentsY").getNodeValue()));
			randomAreaPrototype.setSegmentsY(Integer.parseInt(randomAreaNode
							.getAttributes().getNamedItem("segmentsY")
							.getNodeValue()));
			
			NodeList configurationElements = randomAreaNode.getChildNodes();

			for (int j = 0; j < configurationElements.getLength(); j++) {
				Node configNode = configurationElements.item(j);
				String name = configNode.getLocalName();

				if ("defaultWallCell".equals(name)) {
					randomAreaPrototype.setWallPrototype(parseCellNode(getChildNode(configNode, "cell")));
				} else if ("defaultFloorCell".equals(name)) {
					randomAreaPrototype.setFloorPrototype(parseCellNode(getChildNode(configNode, "cell")));
				} else if ("portalSegment".equals(name)) {
					
					MatrixPortalSegment portalSegment = new MatrixPortalSegment();
					
					portalSegment.setPortal(configNode.getAttributes().getNamedItem("portal").getNodeValue());
					portalSegment.setRemoteArea(configNode.getAttributes().getNamedItem("remoteArea").getNodeValue());
					portalSegment.setRemotePortal(configNode.getAttributes().getNamedItem("remotePortal").getNodeValue());
					
					portalSegment.setEastConnector(SegmentConnector.TUNNEL);
					portalSegment.setWestConnector(SegmentConnector.TUNNEL);
					portalSegment.setNorthConnector(SegmentConnector.TUNNEL);
					portalSegment.setSouthConnector(SegmentConnector.TUNNEL);
					
					portalSegment.setMap(new int[][] {
							{0,0,0,0,1,0,0,0,0},
							{0,1,1,0,1,0,1,1,0},
							{0,1,1,1,1,1,1,1,0},
							{0,0,1,1,1,1,1,0,0},
							{1,1,1,1,1,1,1,1,1},
							{0,0,1,1,1,1,1,0,0},
							{0,1,1,1,1,1,1,1,0},
							{0,1,1,0,1,0,1,1,0},
							{0,0,0,0,1,0,0,0,0}
						});
					
					portalSegment.setPortalCell(parseCellNode(getChildNode(getChildNode(configNode, "portalCell"), "cell")));

					randomAreaPrototype.getPrebuiltSegments().add(portalSegment);
					
				} else if ("population".equals(name)) {
					NodeList pupulationNodes = configNode.getChildNodes();
					
					for (int k = 0; k < pupulationNodes.getLength(); k++) {
						Node populationNode = pupulationNodes.item(k);
						String populationNodesName = populationNode.getLocalName();
						
						if ("actor".equals(populationNodesName)) {
							
							String actorKey = populationNode.getAttributes().getNamedItem("key").getNodeValue();
							String actorDensity = populationNode.getAttributes().getNamedItem("density").getNodeValue();
							
							ActorSpawn actorSpawn = new ActorSpawn();
							actorSpawn.setActorKey(actorKey);
							actorSpawn.setDensity(Integer.parseInt(actorDensity));
							randomAreaPrototype.getSpawns().add(actorSpawn);
							
						}
					}
						
				}

			}
			
		}
		
		
	}

	private void parsePaintedAreas(NodeList areaDefinitions) {
		
		for (int i = 0; i < areaDefinitions.getLength(); i++) {
			Node areaDefinition = areaDefinitions.item(i);

			PaintedAreaPrototype prototype = new PaintedAreaPrototype();
			String prototypeKey = areaDefinition.getAttributes()
					.getNamedItem("id").getNodeValue();

			this.getPrototypes().put(prototypeKey, prototype);

			prototype.setWidth(Integer.parseInt(areaDefinition
					.getAttributes().getNamedItem("width").getNodeValue()));
			prototype
					.setHeight(Integer.parseInt(areaDefinition
							.getAttributes().getNamedItem("height")
							.getNodeValue()));

			NodeList brushes = areaDefinition.getChildNodes();

			for (int j = 0; j < brushes.getLength(); j++) {

				Node brush = brushes.item(j);

				AreaPainter brushInstance = null;

				String name = brush.getNodeName();

				if ("cellFill".equals(name)) {

					brushInstance = new CellFill();

					parseBrush(brushInstance, brush);

				} else if ("cellRandomize".equals(name)) {

					CellRandomize randomize = new CellRandomize();
					brushInstance = randomize;
					randomize.setDensity(Integer.parseInt(brush
							.getAttributes().getNamedItem("density")
							.getNodeValue()));

					parseBrush(brushInstance, brush);

				} else if ("cellLine".equals(name)) {

					brushInstance = new CellLine();

					parseBrush(brushInstance, brush);

				} else if ("cellPaint".equals(name)) {

					brushInstance = new CellFill();

					parseBrush(brushInstance, brush);

				} else {

					continue;

				}

				prototype.getDrawList().add(brushInstance);

			}

		}
		
	}

	private void parseBrush(AreaPainter brushInstance, Node brush) {

		brushInstance.setX(Integer.parseInt(brush.getAttributes().getNamedItem(
				"x").getNodeValue()));
		if (brush.getAttributes().getNamedItem("toX") != null) {
			brushInstance.setToX(Integer.parseInt(brush.getAttributes()
					.getNamedItem("toX").getNodeValue()));
		} else {
			brushInstance.setToX(brushInstance.getX());
		}
		brushInstance.setY(Integer.parseInt(brush.getAttributes().getNamedItem(
				"y").getNodeValue()));
		if (brush.getAttributes().getNamedItem("toY") != null) {
			brushInstance.setToY(Integer.parseInt(brush.getAttributes()
					.getNamedItem("toY").getNodeValue()));
		} else {
			brushInstance.setToY(brushInstance.getY());
		}

		NodeList brushElements = brush.getChildNodes();

		for (int i = 0; i < brushElements.getLength(); i++) {
			Node brushElement = brushElements.item(i);
			String name = brushElement.getLocalName();

			if ("cell".equals(name)) {
				brushInstance.setPrototypeCell(parseCellNode(brushElement));
			} else if ("actor".equals(name)) {
				brushInstance.setActorKey(brushElement.getAttributes()
						.getNamedItem("key").getNodeValue());
			} else if ("item".equals(name)) {
				brushInstance.setItemKey(brushElement.getAttributes()
						.getNamedItem("key").getNodeValue());
			} else if ("portal".equals(name)) {
				AreaPortal prototypePortal = new AreaPortal();
				prototypePortal.setRemoteArea(brushElement.getAttributes()
						.getNamedItem("remoteArea").getNodeValue());
				prototypePortal.setRemotePortal(brushElement.getAttributes()
						.getNamedItem("remotePortal").getNodeValue());
				brushInstance.setPrototypePortal(prototypePortal);
			} else if ("portalTarget".equals(name)) {
				AreaPortal prototypePortal = new AreaPortal();
				prototypePortal.setThisPortalKey(brushElement.getAttributes()
						.getNamedItem("key").getNodeValue());
				brushInstance.setPrototypePortal(prototypePortal);
			}

		}

	}
	
	private Cell parseCellNode(Node cellNode) {
		Cell prototypeCell = new Cell();
		
		Representation graphics = new Representation();

		graphics.setModelKey(cellNode.getAttributes().getNamedItem(
				"graphicsKey").getNodeValue());
		String color = cellNode.getAttributes().getNamedItem(
				"graphicsColor").getNodeValue();

		graphics.setModelColor(colorParser.parse(color));

		prototypeCell.setGraphics(graphics);

		prototypeCell.setMoveThrough(booleanParser.parse(cellNode
				.getAttributes().getNamedItem("moveThrough")
				.getNodeValue()));
		prototypeCell.setSeeThrough(booleanParser.parse(cellNode
				.getAttributes().getNamedItem("seeThrough")
				.getNodeValue()));
		prototypeCell.setDescription(cellNode.getAttributes()
				.getNamedItem("description").getNodeValue());
		
		return prototypeCell;
	}

	public static AreaFactory getInstance() {
		if (instance == null) {
			instance = new AreaFactory();
		}
		return instance;
	}

	public Area getArea(Model model, String key) {

		Area result = this.getAreas().get(key);

		if (result == null) {
			AreaPrototype prototype = this.getPrototypes().get(key);
			result = prototype.buildArea();
			this.getAreas().put(key, result);
		}

		result.setModel(model);

		return result;
	}

	private Map<String, Area> getAreas() {
		return this.areas;
	}

	private void setAreas(Map<String, Area> areas) {
		this.areas = areas;
	}

	private Map<String, AreaPrototype> getPrototypes() {
		return this.prototypes;
	}

	private void setPrototypes(Map<String, AreaPrototype> prototypes) {
		this.prototypes = prototypes;
	}

}
