package foh.rpg.area.factory.randomizer;

import java.util.LinkedList;
import java.util.List;

public class SegmentEnumenator {
	
	private List<Segment> segments;
	
	private static SegmentEnumenator instance;
	
	public static SegmentEnumenator getInstance() {
		if (instance == null) {
			instance = new SegmentEnumenator();
		}
		return instance;
	}
	
	private SegmentEnumenator() {
		
		this.segments = new LinkedList<Segment>();
		
		//1) Empty MatrixSegment
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}));
		
		//2) Dead ends
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{1,1,1,1,1,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setWestConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setWestConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setNorthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setNorthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setSouthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setSouthConnector(SegmentConnector.HALLWAY));

		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,1,1,1,1,1},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setEastConnector(SegmentConnector.HALLWAY));
		
		// dead ends with rooms
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setWestConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,1,1,1,1,1,1,1,0},
				{0,1,1,1,1,1,1,1,0},
				{0,1,1,1,1,1,1,1,0},
				{1,1,1,1,1,1,1,1,0},
				{0,1,1,1,1,1,1,1,0},
				{0,1,1,1,1,1,1,1,0},
				{0,1,1,1,1,1,1,1,0},
				{0,0,0,0,0,0,0,0,0}
			}).setWestConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,1,1,1,1,1,1,1,0},
				{0,1,1,1,1,1,1,1,0},
				{0,1,1,0,0,0,1,1,0},
				{1,1,1,0,1,1,1,1,0},
				{0,1,1,0,0,0,1,1,0},
				{0,1,1,1,1,1,1,1,0},
				{0,1,1,1,1,1,1,1,0},
				{0,0,0,0,0,0,0,0,0}
			}).setWestConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
		}).setNorthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
		}).setSouthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
		}).setEastConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,1,1,1,1,1,0,0},
				{1,1,1,1,1,1,1,0,0},
				{1,1,1,1,1,1,1,0,0},
				{1,1,1,1,1,1,1,0,0},
				{0,0,1,1,1,1,1,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setWestConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,1,1,1,1,1,0,0},
				{0,0,1,1,1,1,1,0,0},
				{0,0,1,1,1,1,1,0,0},
				{0,0,1,1,1,1,1,0,0},
				{0,0,1,1,1,1,1,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setNorthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,1,1,1,1,1,0,0},
				{0,0,1,1,1,1,1,0,0},
				{0,0,1,1,1,1,1,0,0},
				{0,0,1,1,1,1,1,0,0},
				{0,0,1,1,1,1,1,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setSouthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,1,1,1,1,1,0,0},
				{0,0,1,1,1,1,1,1,1},
				{0,0,1,1,1,1,1,1,1},
				{0,0,1,1,1,1,1,1,1},
				{0,0,1,1,1,1,1,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setEastConnector(SegmentConnector.HALLWAY));
		
		// "I" connectors
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,1,1,1,1,1,0,0},
				{0,1,1,1,1,1,1,1,0},
				{0,1,1,1,1,1,1,1,0},
				{1,1,1,1,1,1,1,1,1},
				{0,1,1,1,1,1,1,1,0},
				{0,1,1,1,1,1,1,1,0},
				{0,0,1,1,1,1,1,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setNorthConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setEastConnector(SegmentConnector.HALLWAY)
			.setWestConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setNorthConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,1,0,1,0,1,0,1,0},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,1},
				{0,1,0,1,0,1,0,1,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setEastConnector(SegmentConnector.HALLWAY)
			.setWestConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,1,1,1,1,1,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,1,1,1,1,1,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,1,1,1,1,1,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,1,1,1,1,1,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setNorthConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{1,0,0,0,0,0,0,0,1},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setEastConnector(SegmentConnector.HALLWAY)
			.setWestConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,0,1,0,0,0},
				{0,0,0,1,0,1,0,0,0},
				{0,0,0,1,0,1,0,0,0},
				{0,0,0,1,0,1,0,0,0},
				{0,0,0,1,0,1,0,0,0},
				{0,0,0,1,0,1,0,0,0},
				{0,0,0,1,0,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setNorthConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setNorthConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setNorthConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,1,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setEastConnector(SegmentConnector.HALLWAY)
			.setWestConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,1,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.HALLWAY));
		
		// "T" connectors
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.TUNNEL)
			.setNorthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{1,1,1,1,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setNorthConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,1,1,1,1},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setNorthConnector(SegmentConnector.TUNNEL)
			.setEastConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setEastConnector(SegmentConnector.HALLWAY)
			.setWestConnector(SegmentConnector.HALLWAY)
			.setNorthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setEastConnector(SegmentConnector.HALLWAY)
			.setWestConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setNorthConnector(SegmentConnector.HALLWAY)
			.setWestConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setNorthConnector(SegmentConnector.HALLWAY)
			.setEastConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.TUNNEL)
			.setNorthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setEastConnector(SegmentConnector.HALLWAY)
			.setWestConnector(SegmentConnector.HALLWAY)
			.setNorthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,1,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.HALLWAY)
			.setNorthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setEastConnector(SegmentConnector.HALLWAY)
			.setWestConnector(SegmentConnector.TUNNEL)
			.setNorthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{1,1,1,1,1,0,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.HALLWAY)
			.setNorthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,0,1,1,1,1,1},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setEastConnector(SegmentConnector.HALLWAY)
			.setWestConnector(SegmentConnector.TUNNEL)
			.setNorthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setEastConnector(SegmentConnector.HALLWAY)
			.setWestConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,1,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setEastConnector(SegmentConnector.HALLWAY)
			.setWestConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{1,1,1,1,1,0,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,0,1,1,1,1,1},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setEastConnector(SegmentConnector.HALLWAY)
			.setWestConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{1,1,1,1,1,0,0,0,0},
				{1,1,1,1,1,0,0,0,0},
				{1,1,1,1,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setNorthConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{1,1,1,1,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setNorthConnector(SegmentConnector.HALLWAY)
			.setWestConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{1,1,1,1,1,0,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setNorthConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setNorthConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setNorthConnector(SegmentConnector.HALLWAY)
			.setWestConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setNorthConnector(SegmentConnector.HALLWAY)
			.setWestConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setNorthConnector(SegmentConnector.HALLWAY)
			.setEastConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setNorthConnector(SegmentConnector.HALLWAY)
			.setEastConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setNorthConnector(SegmentConnector.TUNNEL)
			.setEastConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,1,1,1,1},
				{0,0,0,0,1,1,1,1,1},
				{0,0,0,0,1,1,1,1,1},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
			}).setNorthConnector(SegmentConnector.TUNNEL)
			.setEastConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setNorthConnector(SegmentConnector.TUNNEL)
			.setEastConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setNorthConnector(SegmentConnector.HALLWAY)
			.setEastConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
		// + connectors
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setNorthConnector(SegmentConnector.TUNNEL)
			.setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setEastConnector(SegmentConnector.HALLWAY)
			.setWestConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.HALLWAY)
			.setNorthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setNorthConnector(SegmentConnector.TUNNEL)
			.setEastConnector(SegmentConnector.HALLWAY)
			.setWestConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.HALLWAY)
			.setNorthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.TUNNEL)
			.setNorthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,0,1,1,1,1,1},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setEastConnector(SegmentConnector.HALLWAY)
			.setWestConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.TUNNEL)
			.setNorthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{1,1,1,1,1,0,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.TUNNEL)
			.setNorthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,1,1,1,1,1,1,1,0},
				{0,1,1,1,1,1,1,1,0},
				{1,1,1,1,1,1,1,1,0},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,0},
				{0,1,1,1,1,1,1,1,0},
				{0,1,1,1,1,1,1,1,0},
				{0,0,0,0,1,0,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.TUNNEL)
			.setNorthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.HALLWAY)
			.setNorthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,1,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.TUNNEL)
			.setNorthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,1,1,1,1,1,0,0},
				{1,1,1,1,1,1,1,0,0},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,1,1,0,0},
				{0,0,1,1,1,1,1,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.TUNNEL)
			.setNorthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{1,1,1,1,1,1,1,1,1},
				{1,1,1,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.HALLWAY)
			.setNorthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,1,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setEastConnector(SegmentConnector.HALLWAY)
			.setWestConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.HALLWAY)
			.setNorthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,1,1,1},
				{1,1,1,1,1,1,1,1,1},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setEastConnector(SegmentConnector.HALLWAY)
			.setWestConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.TUNNEL)
			.setNorthConnector(SegmentConnector.HALLWAY));
		
		// + rooms
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,1,1,1,1,1,1,1,0},
				{0,1,1,1,1,1,1,1,0},
				{0,1,1,1,1,1,1,1,0},
				{1,1,1,1,0,1,1,1,1},
				{0,1,1,1,1,1,1,1,0},
				{0,1,1,1,1,1,1,1,0},
				{0,1,1,1,1,1,1,1,0},
				{0,0,0,0,1,0,0,0,0}
			}).setNorthConnector(SegmentConnector.TUNNEL)
			.setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,1,1,1,1,1,1,1,0},
				{0,1,0,0,0,0,0,1,0},
				{0,1,0,0,0,0,0,1,0},
				{1,1,0,0,0,0,0,1,1},
				{0,1,0,0,0,0,0,1,0},
				{0,1,0,0,0,0,0,1,0},
				{0,1,1,1,1,1,1,1,0},
				{0,0,0,0,1,0,0,0,0}
			}).setNorthConnector(SegmentConnector.TUNNEL)
			.setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,1,1,1,1,1,0,0},
				{0,1,1,1,1,1,1,1,0},
				{0,1,1,1,1,1,1,1,0},
				{1,1,1,1,0,1,1,1,1},
				{0,1,1,1,1,1,1,1,0},
				{0,1,1,1,1,1,1,1,0},
				{0,0,1,1,1,1,1,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setNorthConnector(SegmentConnector.TUNNEL)
			.setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,1,1,1,1,1,0,0},
				{0,1,1,1,1,1,1,1,0},
				{1,1,1,1,1,1,1,1,1},
				{0,1,1,1,1,1,1,1,0},
				{0,0,1,1,1,1,1,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setNorthConnector(SegmentConnector.TUNNEL)
			.setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
		// L connectors
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,1,1,1,1},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL)
			.setNorthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,1,1,1,1,1},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{1,1,1,1,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setWestConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{1,1,1,1,1,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setNorthConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setWestConnector(SegmentConnector.HALLWAY)
			.setNorthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{1,1,1,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setWestConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setEastConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setNorthConnector(SegmentConnector.HALLWAY)
			.setEastConnector(SegmentConnector.HALLWAY));
		
		
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL)
			.setNorthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,1,1,1,1},
				{0,0,0,0,1,1,1,1,1},
				{0,0,0,0,1,1,1,1,1},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setEastConnector(SegmentConnector.HALLWAY)
			.setNorthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,1,1,1,1,1},
				{0,0,0,0,1,1,1,1,1},
				{0,0,0,0,1,1,1,1,1},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setEastConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,1,1,1,1,1,1},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{1,1,1,1,1,0,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0}
			}).setWestConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.HALLWAY));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{1,1,1,1,1,0,0,0,0},
				{1,1,1,1,1,0,0,0,0},
				{1,1,1,1,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0}
			}).setWestConnector(SegmentConnector.HALLWAY)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{0,0,0,1,1,1,0,0,0},
				{1,1,1,1,1,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setNorthConnector(SegmentConnector.HALLWAY)
			.setWestConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{0,0,0,0,1,0,0,0,0},
				{1,1,1,1,1,0,0,0,0},
				{1,1,1,1,1,0,0,0,0},
				{1,1,1,1,1,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0,0}
			}).setNorthConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.HALLWAY));
		
		
		// misc
		
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,0,0,0,0,0},
				{0,1,1,1,0,1,1,1,0},
				{0,1,1,1,0,1,1,1,0},
				{0,0,1,0,0,0,1,0,0},
				{1,1,1,1,1,1,1,1,1},
				{0,0,1,0,0,0,1,0,0},
				{0,1,1,1,0,1,1,1,0},
				{0,1,1,1,0,1,1,1,0},
				{0,0,0,0,0,0,0,0,0}
			}).setEastConnector(SegmentConnector.TUNNEL)
			.setWestConnector(SegmentConnector.TUNNEL));
		
		segments.add(new MatrixSegment().setMap(new int[][] {
				{0,0,0,0,1,0,0,0,0},
				{0,1,1,0,1,0,1,1,0},
				{0,1,1,1,1,1,1,1,0},
				{0,1,1,0,1,0,1,1,0},
				{0,0,0,0,1,0,0,0,0},
				{0,1,1,0,1,0,1,1,0},
				{0,1,1,1,1,1,1,1,0},
				{0,1,1,0,1,0,1,1,0},
				{0,0,0,0,1,0,0,0,0}
			}).setNorthConnector(SegmentConnector.TUNNEL)
			.setSouthConnector(SegmentConnector.TUNNEL));
		
	}
	
	public List<Segment> getSegmentsFor(TemplateSegment template) {
		
		List<Segment> result = new LinkedList<Segment>();
		
		for (Segment segment : segments) {
			
			if ( template.matches(segment) ) {
				
				result.add(segment);
				
			}
				
			
		}
		
		return result;
		
	}

}
