package foh.rpg.character;

import foh.rpg.character.skill.SkillType;
import foh.rpg.item.magical.AtomicProperty;
import foh.rpg.item.magical.BonusType;
import foh.rpg.item.magical.MagicalProperty;

public enum Ability {
	
	HEAL {
		@Override
		public SkillType getLinkedSkill() {
			return SkillType.HEALING_MAGIC;
		}

		@Override
		public MagicalProperty getMagicalProperty(int linkedSkillLevel) {
			MagicalProperty magic = new MagicalProperty();
			AtomicProperty property = new AtomicProperty();
			property.setAmount(linkedSkillLevel);
			property.setType(BonusType.HEALTH_RESTORE);
			magic.getProperties().add(property);
			return magic;
		}

		@Override
		public String getName() {
			return "Heal";
		}

		@Override
		public Target getTarget() {
			return Target.SELF;
		}

		@Override
		public AbilityCost getCost() {
			AbilityCost cost = new AbilityCost();
			cost.setMpCost(10);
			cost.setApCost(3);
			return cost;
		}
	},
	INFERNO {
		@Override
		public SkillType getLinkedSkill() {
			return SkillType.FIRE_MAGIC;
		}

		@Override
		public MagicalProperty getMagicalProperty(int linkedSkillLevel) {
			MagicalProperty magic = new MagicalProperty();
			AtomicProperty property = new AtomicProperty();
			property.setAmount(linkedSkillLevel);
			property.setType(BonusType.FIRE_DAMAGE);
			magic.getProperties().add(property);
			return magic;
		}

		@Override
		public String getName() {
			return "Inferno";
		}

		@Override
		public Target getTarget() {
			return Target.SELF;
		}

		@Override
		public AbilityCost getCost() {
			AbilityCost cost = new AbilityCost();
			cost.setMpCost(10);
			cost.setApCost(2);
			return cost;
		}
	},
	VAMPIRIC_BURST {
		@Override
		public SkillType getLinkedSkill() {
			return SkillType.NECROMANCY;
		}

		@Override
		public MagicalProperty getMagicalProperty(int linkedSkillLevel) {
			MagicalProperty magic = new MagicalProperty();
			AtomicProperty property = new AtomicProperty();
			property.setAmount(linkedSkillLevel);
			property.setType(BonusType.AOE_ENERGYSTEAL);
			magic.getProperties().add(property);
			
			AtomicProperty property2 = new AtomicProperty();
			property2.setAmount(linkedSkillLevel);
			property2.setType(BonusType.AOE_LIFESTEAL);
			magic.getProperties().add(property2);
			
			return magic;
		}

		@Override
		public String getName() {
			return "Vampiric Burst";
		}

		@Override
		public Target getTarget() {
			return Target.SELF;
		}

		@Override
		public AbilityCost getCost() {
			AbilityCost cost = new AbilityCost();
			cost.setMpCost(25);
			cost.setApCost(1);
			return cost;
		}
	},
	LIGHTNING_BOLT {

		@Override
		public SkillType getLinkedSkill() {
			return SkillType.STORM_MAGIC;
		}

		@Override
		public MagicalProperty getMagicalProperty(int linkedSkillLevel) {
			MagicalProperty magic = new MagicalProperty();
			AtomicProperty property = new AtomicProperty();
			property.setAmount(linkedSkillLevel);
			property.setType(BonusType.LIGHTNING_DAMAGE);
			magic.getProperties().add(property);
			return magic;
		}

		@Override
		public String getName() {
			return "Lightning Bolt";
		}

		@Override
		public Target getTarget() {
			return Target.ACTOR;
		}

		@Override
		public AbilityCost getCost() {
			AbilityCost cost = new AbilityCost();
			cost.setMpCost(5);
			cost.setApCost(1);
			return cost;
		}
		
	},
	BLESS{

		@Override
		public AbilityCost getCost() {
			AbilityCost cost = new AbilityCost();
			cost.setMpCost(5);
			cost.setApCost(1);
			return cost;
		}

		@Override
		public SkillType getLinkedSkill() {
			return SkillType.HEALING_MAGIC;
		}

		@Override
		public MagicalProperty getMagicalProperty(int linkedSkillLevel) {
			MagicalProperty magic = new MagicalProperty();
			AtomicProperty property = new AtomicProperty();
			property.setAmount(linkedSkillLevel);
			property.setType(BonusType.BLESS);
			magic.getProperties().add(property);
			return magic;
		}

		@Override
		public String getName() {
			return "Bless";
		}

		@Override
		public Target getTarget() {
			return Target.SELF;
		}
		
	},
	WITHER {

		@Override
		public AbilityCost getCost() {
			AbilityCost cost = new AbilityCost();
			cost.setMpCost(10);
			cost.setApCost(5);
			return cost;
		}

		@Override
		public SkillType getLinkedSkill() {
			return SkillType.NECROMANCY;
		}

		@Override
		public MagicalProperty getMagicalProperty(int linkedSkillLevel) {
			MagicalProperty magic = new MagicalProperty();
			AtomicProperty property = new AtomicProperty();
			property.setAmount(linkedSkillLevel);
			property.setType(BonusType.HEALTH_DRAIN);
			magic.getProperties().add(property);
			return magic;
		}

		@Override
		public String getName() {
			return "Wither";
		}

		@Override
		public Target getTarget() {
			return Target.ACTOR;
		}
	},
	BLIZZARD {

		@Override
		public AbilityCost getCost() {
			AbilityCost cost = new AbilityCost();
			cost.setMpCost(10);
			cost.setApCost(5);
			return cost;
		}

		@Override
		public SkillType getLinkedSkill() {
			return SkillType.STORM_MAGIC;
		}

		@Override
		public MagicalProperty getMagicalProperty(int linkedSkillLevel) {
			MagicalProperty magic = new MagicalProperty();
			AtomicProperty property = new AtomicProperty();
			property.setAmount(linkedSkillLevel);
			property.setType(BonusType.ICE_DAMAGE);
			magic.getProperties().add(property);
			return magic;
		}

		@Override
		public String getName() {
			return "Blizzard";
		}

		@Override
		public Target getTarget() {
			return Target.TILE;
		}
		
	};
	
	public abstract String getName();
	public abstract SkillType getLinkedSkill();
	public abstract MagicalProperty getMagicalProperty(int linkedSkillLevel);
	public abstract Target getTarget();
	public abstract AbilityCost getCost();
	
}
