package foh.rpg.character;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import foh.rpg.area.Area;
import foh.rpg.area.Cell;
import foh.rpg.area.SpawnPoint;
import foh.rpg.character.attribute.Attribute;
import foh.rpg.character.attribute.AttributeList;
import foh.rpg.character.attribute.AttributeType;
import foh.rpg.character.damage.DamageChunk;
import foh.rpg.character.damage.DamagePacket;
import foh.rpg.character.damage.DamageType;
import foh.rpg.character.effect.EffectStack;
import foh.rpg.character.factory.ActorPrototype;
import foh.rpg.character.npc.AI;
import foh.rpg.character.skill.Skill;
import foh.rpg.character.skill.SkillList;
import foh.rpg.character.skill.SkillType;
import foh.rpg.character.xp.XPPacket;
import foh.rpg.gui.Representation;
import foh.rpg.item.Item;
import foh.rpg.item.ItemRoles;
import foh.rpg.model.Model;
import foh.rpg.util.Dice;

/**
 * @author Zweistein
 *
 */
public class Actor implements Targetable {
	
	private SpawnPoint spawnPoint;
	private ActorPrototype prototype;
	private Area currentArea;
	private Model currentModel;
	private SkillList skills = new SkillList();
	private List<Ability> abilities = new ArrayList<Ability>();
	private AttributeList attributes = new AttributeList();
	private EffectStack effectStack;
	private int unclaimedXP;
	
	private int HP = 0;
	private int MP = 0;
	private int totalHP = 0;
	private int totalMP = 0;
	
	private int totalActionPoints = 0;
	private int actionPoints = 0;

	private String name = null;
	private Representation graphics;
	
	private Inventory inventory = null;
	
	private AI ai;
	
	private Faction faction;
	
	public Faction getFaction() {
		return this.faction;
	}

	public void setFaction(Faction faction) {
		this.faction = faction;
	}

	public int getTotalActionPoints() {
		return this.totalActionPoints + this.getAttributes().getAttribute(AttributeType.AGILITY).getCombatValue();
	}

	public void setTotalActionPoints(int totalActionPoints) {
		this.totalActionPoints = totalActionPoints;
	}

	public int getActionPoints() {
		return this.actionPoints;
	}

	public void setActionPoints(int actionPoints) {
		this.actionPoints = actionPoints;
	}

	public List<Ability> getAbilities() {
		return this.abilities;
	}

	public void setAbilities(List<Ability> abilities) {
		this.abilities = abilities;
	}

	/**
	 * @return Returns the hP.
	 */
	public int getHP() {
		return this.HP;
	}
	
	/**
	 * @param hp The hP to set.
	 */
	public void setHP(int hp) {
		this.HP = hp;
	}
	
	/**
	 * @return Returns the inventory.
	 */
	public Inventory getInventory() {
		return this.inventory;
	}
	
	/**
	 * @param inventory The inventory to set.
	 */
	public void setInventory(Inventory inventory) {
		this.inventory = inventory;
	}
	
	/**
	 * @return Returns the mP.
	 */
	public int getMP() {
		return this.MP;
	}
	
	/**
	 * @param mp The mP to set.
	 */
	public void setMP(int mp) {
		this.MP = mp;
	}
	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return this.name;
	}
	/**
	 * @param name The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * 
	 */
	public Actor() {
		super();
		this.inventory = new Inventory();
		this.setEffectStack(new EffectStack());
	}
	
	public void melee(Actor target) {
		
		if (new Dice(0,100).roll() > target.getBlock()) {
			this.onHit(target);
			target.processDamagePacket(getMeleeDamagePacket());
			
			XPPacket meleeStrengthXP = new XPPacket();
			meleeStrengthXP.getAttributeHiXP().put(AttributeType.STRENGTH, 10);
			this.processXPPacket(meleeStrengthXP);
			
		} else {
			this.getCurrentModel().getMessageBuffer().addLast(target.getName() + " blocks attack from " + this.getName() + ".");
			
			XPPacket shieldBlockXP = new XPPacket();
			shieldBlockXP.getSkillXP().put(SkillType.SHILED_MASTERY, 10);
			shieldBlockXP.getAttributeHiXP().put(AttributeType.AGILITY, 10);
			target.processXPPacket(shieldBlockXP);
			
		}
		
	} 
	
	public void onHit(Actor target) {
		
		for (Slot slot : this.getInventory().getSlots()) {
			if (slot.getSlotContent()!= null) {
				slot.getSlotContent().onHit(this, target);
			}
		}
		
	}
	
	public DamagePacket getMeleeDamagePacket() {
		DamagePacket packet = new DamagePacket();
		packet.setOriginator(this);
		
		// we iterate over each appendace
		// and get attack damage
		
		for (Slot slot : this.getInventory().getSlots()) {
			if (slot.getSlotType() == SlotType.HAND) {
				if (slot.getSlotContent()!= null) {
					
					// each weapon?
					if (slot.getSlotContent().getItemRoles().contains(ItemRoles.WEAPON)) {
						DamagePacket itemPacket = slot.getSlotContent().generateDamagePacket(this);
						packet.getDamageChunks().addAll(itemPacket.getDamageChunks());
					}
					
				} else {
					// empty hand, we assume H-t-H
					DamageChunk chunk = new DamageChunk();
					chunk.setDamageAmount(1 + this.getSkills().getSkill(SkillType.UNARMED_MASTERY).getCombatValue());
					chunk.setDamageType(DamageType.PHYSICAL_DAMAGE);
					chunk.setOriginatingSkill(SkillType.UNARMED_MASTERY);
					packet.getDamageChunks().add(chunk);
				}
			}
		}
		
		this.effectStack.onEffectMeleeDamagePacketGeneration(packet);
		
		return packet;
	}
	
	public void processDamagePacket(DamagePacket damage) {
		
		int recievedDamage = damage.getTotalDamage();
		
		recievedDamage -= this.getAR();
		
		if (recievedDamage < 0) {
			recievedDamage = 0;
		}
		
		XPPacket armorXP = new XPPacket();
		armorXP.getSkillXP().put(SkillType.HEAVY_ARMOR_MASTERY, 10);
		armorXP.getAttributeHiXP().put(AttributeType.CONSTITUTION, 10);
		this.processXPPacket(armorXP);
		
		this.getCurrentModel().getMessageBuffer().addLast(
				damage.getOriginator().getName() + " hits " + this.getName()
						+ " for " + recievedDamage + ".");
		
		//we now award experience from packet to its originator
		
		damage.processXP();
		
		this.HP -= recievedDamage;
		
		if ( this.HP <= 0 ) {
			this.die();
			XPPacket xp = new XPPacket();
			xp.setRawXP(100);
			this.getCurrentModel().getMessageBuffer().addLast(damage.getOriginator().getName()+" gains "+100+"xp.");
			damage.getOriginator().processXPPacket(xp);
		}
		
		// TODO: use resists
	}
	
	/**
	 * 
	 */
	private void die() {
		this.getCurrentModel().getMessageBuffer().addLast(this.getName()+" died.");
		
		for (int i=0; i < this.getInventory().getSlots().size(); i++) {
			if (this.getInventory().getSlots().get(i).getSlotContent() != null) {
				this.getInventory().unEquipItem(i);
			}
		}
		
		List<Item> items = this.getInventory().getBackpack();
		for (Iterator<Item> it = items.iterator(); it.hasNext();) {
			Item item = it.next();
			this.currentArea.getActorTile(this).getItems().add(item);
			it.remove();
		}
		
		Cell tile = this.currentArea.getActorTile(this);
		if (tile != null) {
			tile.setActor(null);
		}
		
		// clear target to get rid of monster.
		if (this.currentModel.getSelectedTarget() == this) {
			this.currentModel.setSelectedTarget(null);
		}
		
		// notify respawn point
		
		this.getSpawnPoint().notifyDeath(this);
		
	}

	/**
	 * @return Returns the totalHP.
	 */
	public int getTotalHP() {
		return this.totalHP + this.getAttributes().getAttribute(AttributeType.CONSTITUTION).getCombatValue();
	}
	
	/**
	 * @param totalHP The totalHP to set.
	 */
	public void setTotalHP(int totalHP) {
		this.totalHP = totalHP;
	}
	
	/**
	 * @return Returns the totalMP.
	 */
	public int getTotalMP() {
		return this.totalMP + this.getAttributes().getAttribute(AttributeType.INTELLECT).getCombatValue();
	}
	
	/**
	 * @param totalMP The totalMP to set.
	 */
	public void setTotalMP(int totalMP) {
		this.totalMP = totalMP;
	}
	
	public int getLOSDistance() {
		return 10;
	}
	
	public Representation getGraphics() {
		return this.graphics;
	}
	
	public void setGraphics(Representation graphics) {
		this.graphics = graphics;
	}
	
	public Area getCurrentArea() {
		return this.currentArea;
	}
	public void setCurrentArea(Area currentArea) {
		this.currentArea = currentArea;
	}
	
	public Model getCurrentModel() {
		return this.currentModel;
	}
	
	public void setCurrentModel(Model currentModel) {
		this.currentModel = currentModel;
	}
	
	public void processXPPacket(XPPacket xp) {
		
		// first, we add leftover XP from last time.
		xp.setRawXP(xp.getRawXP() + this.unclaimedXP);
		this.unclaimedXP = 0;
		
		List<Skill> markedSkills = this.skills.getMarkedSkills();
		List<Attribute> markedLoAttributes = this.attributes.getMarkedLoAttributes();
		List<Attribute> markedHiAttributes = this.attributes.getMarkedHiAttributes();

		int totalMarked = 0;
		
		totalMarked += markedSkills.size();
		totalMarked += markedLoAttributes.size();
		totalMarked += markedHiAttributes.size();

		//if (totalMarked > 0) {
			
		
			int oneXPPiece = 0;
			if (totalMarked > 0) {
				oneXPPiece = xp.getRawXP()/totalMarked;
			}
			// leftover XP after fractioning.
			this.unclaimedXP = xp.getRawXP() - totalMarked * oneXPPiece;
			xp.setRawXP(0);
			
			// for marked skills, we add them to packet
			for (Skill skill : markedSkills) {
				Integer skillXP = xp.getSkillXP().get(skill.getType());
				if (skillXP == null) {
					skillXP = 0;
				}
				skillXP+=oneXPPiece;
				xp.getSkillXP().put(skill.getType(), skillXP);
			}
			this.skills.processXPPacket(xp);
			
			for (Attribute attribute : markedLoAttributes) {
				Integer attributeXP = xp.getAttributeLoXP().get(attribute.getType());
				if (attributeXP == null) {
					attributeXP = 0;
				}
				attributeXP+=oneXPPiece;
				xp.getAttributeLoXP().put(attribute.getType(), attributeXP);
			}
			for (Attribute attribute : markedHiAttributes) {
				Integer attributeXP = xp.getAttributeHiXP().get(attribute.getType());
				if (attributeXP == null) {
					attributeXP = 0;
				}
				attributeXP+=oneXPPiece;
				xp.getAttributeHiXP().put(attribute.getType(), attributeXP);
			}
			this.attributes.processXPPacket(xp);
			
			//in case of overflows, we have some unclaimed XP
			this.unclaimedXP += xp.getRawXP();
		//} else {
			
			// nothing is marked, XP is therefore added to pool of unclaimed XP.
		//	this.unclaimedXP+=xp.getRawXP();
		//}
		
	}
	
	public void setUnclaimedXP(int unclaimedXP) {
		this.unclaimedXP = unclaimedXP;
	}
	
	public SkillList getSkills() {
		return this.skills;
	}
	
	public AttributeList getAttributes() {
		return this.attributes;
	}
	
	public int getUnclaimedXP() {
		return this.unclaimedXP;
	}
	
	public AI getAI() {
		return this.ai;
	}
	
	public void setAI(AI ai) {
		this.ai = ai;
		this.ai.setSelf(this);
	}

	public int getAR() {
		int totalAR = 0;

		for (Slot slot : this.getInventory().getSlots()) {
			if (slot.getSlotContent() != null) {
				totalAR += slot.getSlotContent().getDefence();
				totalAR += this.getSkills().getSkill(slot.getSlotContent().getGoverningSkill()).getCombatValue();
			}
		}
		
		return totalAR;
	}
	
	public int getBlock() {
		int totalBlock = 0;

		for (Slot slot : this.getInventory().getSlots()) {
			if (slot.getSlotContent() != null) {
				totalBlock += slot.getSlotContent().getBlock();
				totalBlock += this.getSkills().getSkill(SkillType.SHILED_MASTERY).getCombatValue();
			}
		}
		
		totalBlock += this.getAttributes().getAttribute(AttributeType.AGILITY).getCombatValue();
		
		return totalBlock;
	}

	public boolean validateCost(AbilityCost ablityCost) {
		
		if (this.getMP() < ablityCost.getMpCost()) {
			return false;
		} else if (this.getHP() < ablityCost.getHealthCost()) {
			return false;
		} else if (this.getActionPoints() <= 0 && ablityCost.getApCost() > 0) {
			// aps can go to negative, but we need more than zero to act
			return false;
		}
		
		return true;
	}

	public void processCostPacket(AbilityCost ablityCost) {
		this.HP -= ablityCost.getHealthCost();
		this.MP -= ablityCost.getMpCost();
		this.actionPoints -= ablityCost.getApCost();
	}

	public void setEffectStack(EffectStack effectStack) {
		this.effectStack = effectStack;
	}

	public EffectStack getEffectStack() {
		return effectStack;
	}

	public void tick() {
		// ap regeneration
		
		if (this.getTotalActionPoints() > this.getActionPoints()) {
			this.setActionPoints(this.getActionPoints()+1);
		}
		
		this.getEffectStack().tick();
	}

	public void setSpawnPoint(SpawnPoint spawnPoint) {
		this.spawnPoint = spawnPoint;
	}

	public SpawnPoint getSpawnPoint() {
		return spawnPoint;
	}

	public void setPrototype(ActorPrototype prototype) {
		this.prototype = prototype;
	}

	public ActorPrototype getPrototype() {
		return prototype;
	}
	
}
