package foh.rpg.character;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import foh.rpg.gui.Gui;
import foh.rpg.item.Item;

/**
 * @author Zweistein
 */
public class Inventory {
	
	private List<Item> backpack = new ArrayList<Item>();
	
	private List<Slot> slots = new ArrayList<Slot>();

	/**
	 * 
	 */
	public Inventory() {
		super();
		this.slots.add(new Slot(SlotType.HEAD_ARMOR,"Head"));
		this.slots.add(new Slot(SlotType.BODY_ARMOR,"Body"));
		this.slots.add(new Slot(SlotType.HAND,"Left Hand"));
		this.slots.add(new Slot(SlotType.HAND,"Right Hand"));
	}

	/**
	 * @param inventory The inventory to set.
	 */
	public void setBackpack(List<Item> inventory) {
		this.backpack = inventory;
	}
	
	public void addItem(Item item) {
		this.backpack.add(item);
	}

	public Collection<Item> getAllItems() {
		
		ArrayList<Item> tmp = new ArrayList<Item>();
		
		tmp.addAll(this.getBackpack());
		
		for (Iterator<Slot> it = this.slots.iterator(); it.hasNext();) {
			Slot slot = it.next();
			
			if (slot.getSlotContent() != null) {
				tmp.add(slot.getSlotContent());				
			}
			
		}
		
		return tmp;
	}

	public List<Slot> getSlots() {
		return this.slots;
	}

	public List<Item> getBackpack() {
		return this.backpack;
	}
	
	public Slot getSlotByKey(char key) {
		Slot tmp = null;
		int index = key-'a';
		
		if (index>=0 && index <= this.getSlots().size()) {	
			tmp = this.getSlots().get(index);
		}
		
		return tmp;
	}
	
	public void equipItem(int backpackKey, int slotKey) {
		
		Slot targetSlot = this.getSlots().get(slotKey);
		Item item = this.getBackpack().get(backpackKey);
		
		if (targetSlot != null) {
			if (targetSlot.getSlotContent() == null) {
				if (item.getAllowedSlotTypes().contains(targetSlot.getSlotType())) {
						targetSlot.setSlotContent(item);
						this.getBackpack().remove(backpackKey);
				} else {
					Gui.getConsole().printTextLn("Wrong Slot.");
				}
			} else {
				Gui.getConsole().printTextLn("Slot Not Empty.");
			}
		} else {
			Gui.getConsole().printTextLn("Bad Slot.");
		}
		
	}
	
	public void unEquipItem(int slotKey) {
		
		Slot targetSlot = this.getSlots().get(slotKey);
		
		this.getBackpack().add(targetSlot.getSlotContent());
		targetSlot.setSlotContent(null);
		
	}
	
}
