package foh.rpg.character.attribute;

import foh.rpg.character.xp.XPGainer;
import foh.rpg.character.xp.XPPacket;
import foh.rpg.character.xp.XPToLevelTransator;

public class Attribute implements XPGainer {

	private int xpHi;
	private int xpLo;
	private int combatValue;
	private boolean markedHi;
	private boolean markedLo;
	private AttributeType type;

	public Attribute(AttributeType type) {
		this.type = type;
		this.combatValue = 10;
	}
		

	public int getMaxValue() {
		return 10 + XPToLevelTransator.getLevelFromXP(this.xpHi);
	}

	public int getMinValue() {
		return 10 - XPToLevelTransator.getLevelFromXP(this.xpLo);
	}

	public AttributeType getType() {
		return this.type;
	}

	public boolean isMarkedHi() {
		return this.markedHi;
	}

	public int getCombatValue() {
		return this.combatValue;
	}

	public void setCombatValue(int combatValue) {
		this.combatValue = combatValue;
	}

	public void setMarkedHi(boolean markedHi) {
		this.markedHi = markedHi;
	}

	public boolean isMarkedLo() {
		return this.markedLo;
	}

	public void setMarkedLo(boolean markedLo) {
		this.markedLo = markedLo;
	}

	public void processXPPacket(XPPacket xp) {
		Integer XPForAttHi = xp.getAttributeHiXP().remove(this.getType());
		if (XPForAttHi != null) {
			this.xpHi += XPForAttHi.intValue();

			// if xp overflow over max, we return it to pool of generic xp and
			// turn off
			// mark
			if (this.xpHi > XPToLevelTransator.getMaxXP()) {
				xp.setRawXP(this.xpHi - XPToLevelTransator.getMaxXP());
				this.xpHi = XPToLevelTransator.getMaxXP();
				if (this.isMarkedHi()) {
					this.setMarkedHi(false);
				}
			}
		}
		Integer XPForAttLo = xp.getAttributeLoXP().remove(this.getType());
		if (XPForAttLo != null) {
			this.xpLo += XPForAttLo.intValue();

			// if xp overflow over max, we return it to pool of generic xp and
			// turn off
			// mark
			if (this.xpLo > XPToLevelTransator.getMaxXP()) {
				xp.setRawXP(this.xpLo - XPToLevelTransator.getMaxXP());
				this.xpLo = XPToLevelTransator.getMaxXP();
				if (this.isMarkedLo()) {
					this.setMarkedLo(false);
				}
			}
		}
	}

	@Override
	public String toString() {
		StringBuilder string = new StringBuilder();

		string.append(this.getType()).append(" (");

		if (this.isMarkedLo()) {
			string.append("*) ");
		} else {
			string.append(" ) ");
		}

		string.append(this.getMinValue()).append(" xp:").append(this.xpLo).append("/").append(XPToLevelTransator.getXPForNextLevel(XPToLevelTransator.getLevelFromXP(this.xpLo)));
		
		string.append(" / ").append(
				this.getCombatValue()).append(" / ");
		string.append(this.getMaxValue())
				.append(" (");

		if (this.isMarkedHi()) {
			string.append("*)");
		} else {
			string.append(" )");
		}
		string.append(" xp:").append(this.xpHi).append("/").append(XPToLevelTransator.getXPForNextLevel(XPToLevelTransator.getLevelFromXP(this.xpHi)));

		return string.toString();
	}


	public int getXpHi() {
		return this.xpHi;
	}


	public int getXpLo() {
		return this.xpLo;
	}

}
