package foh.rpg.character.attribute;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import foh.rpg.character.xp.XPGainer;
import foh.rpg.character.xp.XPPacket;

public class AttributeList implements XPGainer {
	
	private Map<AttributeType, Attribute> attributes;
	
	private int attributePoints = 0;
	
	public AttributeList() {
		super();
		this.attributes = new HashMap<AttributeType, Attribute>();
		
		for (AttributeType attributeType : AttributeType.values())  {
			this.attributes.put(attributeType, new Attribute(attributeType));
		}
	}
	
	public List<Attribute> getMarkedLoAttributes() {
		List<Attribute> list = new ArrayList<Attribute>();
		for (AttributeType attributeType : AttributeType.values()) {
			Attribute attribute = this.attributes.get(attributeType) ;
			if (attribute.isMarkedLo()) {
				list.add(attribute);
			}
		}
		return list;
	}
	
	public List<Attribute> getMarkedHiAttributes() {
		List<Attribute> list = new ArrayList<Attribute>();
		for (AttributeType attributeType : AttributeType.values()) {
			Attribute attribute = this.attributes.get(attributeType) ;
			if (attribute.isMarkedHi()) {
				list.add(attribute);
			}
		}
		return list;
	}

	public void processXPPacket(XPPacket xp) {
		for (AttributeType attributeType : AttributeType.values()) {
			Attribute attribute = this.attributes.get(attributeType);
			if (attribute !=  null) {
				attribute.processXPPacket(xp);
				
				this.investTo(attributeType);
			}
		}
	}

	@Override
	public String toString() {
		StringBuilder string = new StringBuilder();
		
		for (Attribute attribute : this.attributes.values()) {
			string.append(attribute).append("\n");
		}
		
		return string.toString();
	}

	public Map<AttributeType, Attribute> getAttributes() {
		return this.attributes;
	}
	
	public void investTo(AttributeType type) {
		
		Attribute attribute = this.getAttributes().get(type);
		
		if (attribute.getCombatValue() < attribute.getMaxValue() && this.getAttributePoints() > 0) {
			attribute.setCombatValue(attribute.getCombatValue()+1);
			this.attributePoints--;
		}
		
	}
	
	public void uninvestTo(AttributeType type) {
		
		Attribute attribute = this.getAttributes().get(type);
		
		if (attribute.getCombatValue() > attribute.getMinValue()) {
			attribute.setCombatValue(attribute.getCombatValue()-1);
			this.attributePoints++;
		}
		
	}

	public void setAttributePoints(int attributePoints) {
		this.attributePoints = attributePoints;
	}

	public int getAttributePoints() {
		return attributePoints;
	}

	public Attribute getAttribute(AttributeType attributeType) {
		return this.attributes.get(attributeType);
	}

}
