package foh.rpg.character.factory;

import java.awt.Color;
import java.util.HashMap;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import foh.rpg.character.Actor;
import foh.rpg.character.Faction;
import foh.rpg.gui.Representation;
import foh.rpg.item.factory.ItemFactory;
import foh.rpg.util.FactoryFromXML;

public class ActorFactory extends FactoryFromXML {
	
	private static ActorFactory instance;
	
	private Map<String, ActorPrototype> prototypes;
	
	public static ActorFactory getInstance() {
		if (instance == null) {
			instance = new ActorFactory();
		}
		return instance;
	}
	
	private ActorFactory() {
		this.prototypes = new HashMap<String, ActorPrototype>();
		
		Document doc = this.parseDocument("bestiary.xml");
		
		NodeList monsterNodes = doc.getElementsByTagName("monster");
		
		for (int i = 0; i < monsterNodes.getLength(); i++) {
			Node monsterNode = monsterNodes.item(i);

			ActorPrototype prototype = new ActorPrototype();
			
			String prototypeKey = monsterNode.getAttributes()
					.getNamedItem("id").getNodeValue();

			this.prototypes.put(prototypeKey, prototype);
			
			prototype.setName(monsterNode.getAttributes()
					.getNamedItem("name").getNodeValue());
			
			NodeList configurationElements = monsterNode.getChildNodes();
			
			for (int j = 0; j < configurationElements.getLength(); j++) {
				Node configNode = configurationElements.item(j);
				String name = configNode.getLocalName();

				if ("faction".equals(name)) {
					
					String faction = configNode.getAttributes().getNamedItem("key").getNodeValue();
					prototype.setFaction(Faction.valueOf(faction));
					
				} else if ("representation".equals(name)) {
					
					String key = configNode.getAttributes().getNamedItem("key").getNodeValue();
					String color = configNode.getAttributes().getNamedItem("color").getNodeValue();
					
					prototype.setRepresentation(new Representation(key, colorParser.parse(color) ));
					
				} else if ("hp".equals(name)) {
					
					String dice = configNode.getAttributes().getNamedItem("dice").getNodeValue();
					
					prototype.setHp(diceParser.parse(dice));
					
				} else if ("ai".equals(name)) {
					
					String passive = configNode.getAttributes().getNamedItem("passive").getNodeValue();
					
					prototype.setPassive(booleanParser.parse(passive));
					
				} else if ("dialogue".equals(name)) {
					
					String dialogue = configNode.getAttributes().getNamedItem("key").getNodeValue();
					
					prototype.setDialogue(dialogue);
					
				} else if ("loot".equals(name)) {

					NodeList lootNodes = configNode.getChildNodes();
					
					for (int k = 0; k < lootNodes.getLength(); k++) {
						Node lootNode = lootNodes.item(k);
						
						String lootNodename = lootNode.getLocalName();

						if ("randomItem".equals(lootNodename)) {
							prototype.getLoot().add(new RandomLootPrototype());
						} else if ("item".equals(lootNodename)) {
							String key = lootNode.getAttributes().getNamedItem("key").getNodeValue();
							
							NamedLootPrototype lootPrototype = new NamedLootPrototype();
							lootPrototype.setKey(key);
							prototype.getLoot().add(lootPrototype);
						}
						
					}
				}
				
			}
					
		}
		
	}
	
	public Actor getActor(String key) {
		ActorPrototype prototype = this.prototypes.get(key);
		return prototype.generateActor();
	}

	public static Actor createCharacter() {

		Actor pc = new Actor();

		pc.setGraphics(new Representation("@", Color.WHITE));
		pc.setHP(1000);
		pc.setMP(1000);
		pc.setTotalActionPoints(8);
		pc.setActionPoints(8);
		pc.setFaction(Faction.PLAYER);
		pc.getSkills().setSkillPoints(20);
		pc.getAttributes().setAttributePoints(5);

		for (int i = 0; i < 5; i++) {
			pc.getInventory().getBackpack().add(
					ItemFactory.getInstance().generateItem());
		}

		return pc;

	}

}
