package foh.rpg.character.factory;

import java.awt.Color;
import java.util.LinkedList;
import java.util.List;

import foh.rpg.character.Actor;
import foh.rpg.character.Faction;
import foh.rpg.character.npc.AI;
import foh.rpg.character.npc.dialogue.DialogueFactory;
import foh.rpg.gui.Representation;
import foh.rpg.util.Dice;

public class ActorPrototype {
	
	private String name;
	private Representation representation;
	private Faction faction;
	private Dice hp;
	private String dialogue;
	
	private List<LootPrototype> loot;
	private boolean passive;
	
	public ActorPrototype() {
		
		this.loot = new LinkedList<LootPrototype>();

		this.hp = new Dice(1, 1);
		this.faction = Faction.FRIENDLY_NPC;
		this.representation = new Representation("@", Color.WHITE);
		
	}

	public Actor generateActor() {

		Actor actor = new Actor();
		actor.setPrototype(this);
		actor.setAI(new AI());
		actor.getAI().setPassive(passive);
		
		actor.setName(this.name);
		actor.setGraphics(this.representation.clone());

		actor.setTotalHP(this.hp.roll());
		actor.setHP(actor.getTotalHP());

		actor.setFaction(this.faction);
		
		if (this.dialogue != null) {
			actor.getAI().setDialogue(DialogueFactory.getInstance().getConversation(this.dialogue));
		}
		
		for (LootPrototype lootPrototype: this.loot) {
			actor.getInventory().addItem(lootPrototype.getItem());
		}
		
		actor.getAI().equipmentCheck();
		
		return actor;
	}

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public Faction getFaction() {
		return this.faction;
	}

	public void setFaction(Faction faction) {
		this.faction = faction;
	}

	public Representation getRepresentation() {
		return this.representation;
	}

	public void setRepresentation(Representation representation) {
		this.representation = representation;
	}

	public Dice getHp() {
		return this.hp;
	}

	public void setHp(Dice hp) {
		this.hp = hp;
	}

	public String getDialogue() {
		return this.dialogue;
	}

	public void setDialogue(String dialogue) {
		this.dialogue = dialogue;
	}

	public List<LootPrototype> getLoot() {
		return this.loot;
	}

	public void setLoot(List<LootPrototype> loot) {
		this.loot = loot;
	}
	
	public boolean isPassive() {
		return this.passive;
	}

	public void setPassive(boolean passive) {
		this.passive = passive;
	}

}
