package foh.rpg.character.npc;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import foh.rpg.area.AreaTools;
import foh.rpg.character.Actor;
import foh.rpg.character.Faction;
import foh.rpg.character.Slot;
import foh.rpg.character.npc.dialogue.Conversation;
import foh.rpg.character.position.Position;
import foh.rpg.item.Item;

public class AI {
	
	private Actor self;

	private boolean passive = false;
	private boolean acted = false;
	
	private Conversation dialogue;
	
	// longterm target
	private Position targetPosition = null;

	public Position getTargetPosition() {
		return this.targetPosition;
	}

	public void setTargetPosition(Position targetPosition) {
		this.targetPosition = targetPosition;
	}

	public Actor getSelf() {
		return this.self;
	}
	
	public void setSelf(Actor self) {
		this.self = self;
	}

	public void setActed(boolean acted) {
		this.acted = acted;
	}

	public boolean getActed() {
		return acted;
	}
	
	public void equipmentCheck() {
		
		// runs to check whether actor has free slots and eventually equip items.
		
		for (Slot slot: this.self.getInventory().getSlots()) {
			
			if (slot.getSlotContent() == null) {
			
				for (Iterator<Item> iterator = this.self.getInventory().getBackpack().iterator(); iterator.hasNext();) {
					Item item = iterator.next();
					
					if (item.getAllowedSlotTypes().contains(slot.getSlotType())) {
						slot.setSlotContent(item);
						iterator.remove();
						break;
					}
					
				}
			
			}
			
		}
		
	}

	public void act() {
		
		if (!this.passive) {
		
			int x = this.self.getCurrentArea().getActorTile(this.self).getPosition().getX();
			int y = this.self.getCurrentArea().getActorTile(this.self).getPosition().getY();
			
			int[][] los = AreaTools.checkLOS(this.self.getCurrentArea().getActorTile(this.self).getPosition(),
					this.self.getCurrentArea(), this.self.getLOSDistance());
	
			if (this.getTargetPosition() == null){
				// some different target, look for closest item and try to pick it up
				
				List<Position> visibleItems = new ArrayList<Position>();
				List<Position> visibleFoes = new ArrayList<Position>();
				List<Position> visibleAllies = new ArrayList<Position>();
				
				for (int iX = 0; iX < los.length; iX++) {
					for (int iY = 0; iY < los[iX].length; iY++) {
						if (los[iX][iY] == 1) {
							if (this.self.getCurrentArea().getTile()[iX][iY].getItems().size() != 0) {
								visibleItems.add(new Position(iX, iY));
							}
							if (this.self.getCurrentArea().getTile()[iX][iY].getActor() != null) {
								Faction visibleCharacterFaction = this.self.getCurrentArea().getTile()[iX][iY].getActor().getFaction();
								if (this.self.getFaction().getFoes().contains(visibleCharacterFaction)) {
									visibleFoes.add(new Position(iX, iY));
								} else {
									visibleAllies.add(new Position(iX, iY));
								}
							}
						}
					}
				}
				
				//TODO: target closest
				
				if (visibleFoes.size() > 0) {
					this.setTargetPosition(visibleFoes.get(0));
				} else if (visibleItems.size() > 0) {
					this.setTargetPosition(visibleItems.get(0));
				} else if (visibleAllies.size() > 0) {
					this.setTargetPosition(visibleAllies.get(0));
				} else {
					// nothing of interest visible
				}
				
			}
			
			if (this.getTargetPosition() != null) {
				// we can see pc position, it is within los or last position
				// we know about it
				// now, we need to path to it
	
				Position nextPosition = AreaTools
						.getNextPathElement(this.self.getCurrentArea(), los, x, y,
								getTargetPosition().getX(), getTargetPosition().getY());
	
				int moveToX = nextPosition.getX();
				int moveToY = nextPosition.getY();
	
				if (this.self.getCurrentArea().getTile()[moveToX][moveToY]
						.isMoveThrough()
						&& this.self.getCurrentArea().getTile()[moveToX][moveToY]
								.getActor() == null) {
					// move to unoccupied tile
					
					this.self.getCurrentModel().moveActor(x, y, moveToX, moveToY);
					
				} else if ((this.self.getCurrentArea().getTile()[moveToX][moveToY]
						.isMoveThrough()
						&& this.self.getCurrentArea().getTile()[moveToX][moveToY]
								.getActor() != null && this.self.getCurrentArea()
						.getTile()[moveToX][moveToY].getActor()
						.getFaction().getFoes().contains(this.self.getFaction()))) {
					// attack target
					
					this.self.melee(this.self.getCurrentArea().getTile()[moveToX][moveToY]
							.getActor());
					
				} else if (moveToX == this.getTargetPosition().getX() &&
					moveToY == this.getTargetPosition().getY() ) {
					// we are done
					// clear target if we reached it
					this.setTargetPosition(null);
					
					// also, pick item if it is present
					
					List<Item> itemsOnGround = this.self.getCurrentArea().getActorTile(this.self).getItems();
					
					if (itemsOnGround.size() > 0) {
						this.self.getInventory().addItem(itemsOnGround.remove(0));
					}
					
				}
					
			}
		
		}
		
	}

	public void setPassive(boolean passive) {
		this.passive = passive;
	}

	public boolean isPassive() {
		return passive;
	}

	public void setDialogue(Conversation dialogue) {
		this.dialogue = dialogue;
	}

	public Conversation getDialogue() {
		return dialogue;
	}

}
