package foh.rpg.character.npc.dialogue;

import java.util.HashMap;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import foh.rpg.util.FactoryFromXML;

public class DialogueFactory extends FactoryFromXML {
	
	private static DialogueFactory instance;
	
	private Map<String, Conversation> dialogues;
	
	private DialogueFactory() {
		
		this.dialogues = new HashMap<String, Conversation>();
		
		Document doc = this.parseDocument("dialogue.xml");
		
		NodeList conversations = doc.getElementsByTagName("coversation");
		
		for (int i = 0; i < conversations.getLength(); i++) {
			Node conversationNode = conversations.item(i);

			Conversation conversation = new Conversation();
			String id = conversationNode.getAttributes().getNamedItem("id").getNodeValue();
			
			this.dialogues.put(id, conversation);
			
			conversation.setInital(conversationNode.getAttributes().getNamedItem("inital").getNodeValue());
			conversation.setFin(conversationNode.getAttributes().getNamedItem("fin").getNodeValue());
			
			NodeList stateNodes = conversationNode.getChildNodes();
			
			for (int j = 0; j < stateNodes.getLength(); j++) {
				
				Node stateNode = stateNodes.item(j);
				
				if ("state".equals(stateNode.getNodeName())) {
				
					State state = new State();
					String stateId = stateNode.getAttributes().getNamedItem("id").getNodeValue();
					
					conversation.getStates().put(stateId, state);
					
					state.setText(stateNode.getAttributes().getNamedItem("text").getNodeValue());
					
					NodeList transitionNodes = stateNode.getChildNodes();
					
					for (int k = 0; k < transitionNodes.getLength(); k++) {
						
						Node transitionNode = transitionNodes.item(k);
						
						if ("transition".equals(transitionNode.getNodeName())) {
							
							Transition transition = new Transition();
							
							state.getTransitions().add(transition);
							
							transition.setText(transitionNode.getAttributes().getNamedItem("text").getNodeValue());
							transition.setTo(transitionNode.getAttributes().getNamedItem("to").getNodeValue());
							
						}
						
					}
				
				}
				
			}
			
		}
		
	}
	
	public static DialogueFactory getInstance() {
		if (instance == null) {
			instance = new DialogueFactory();
		}
		return instance;
	}
	
	public Conversation getConversation(String key) {
		return this.dialogues.get(key);
	}

}
