package foh.rpg.character.skill;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import foh.rpg.character.xp.XPGainer;
import foh.rpg.character.xp.XPPacket;


public class SkillList implements XPGainer {
	
	private Map<SkillType, Skill> skills;
	
	private int skillPoints = 0;

	public SkillList() {
		super();
		this.skills = new HashMap<SkillType, Skill>();
		
		for (SkillType skillType: SkillType.values()) {
			this.skills.put(skillType, new Skill(skillType));
		}
	}
	
	public int getSkillPoints() {
		return this.skillPoints;
	}
	
	public void setSkillPoints(int skillPoints) {
		this.skillPoints = skillPoints;
	}
	
	public List<Skill> getMarkedSkills() {
		List<Skill> list = new ArrayList<Skill>();
		for (SkillType skillType : SkillType.values()) {
			Skill skill = this.skills.get(skillType);
			if (skill.isMarked()) {
				list.add(skill);
			}
		}
		return list;
	}
	
	public Skill getSkill(SkillType type) {
		return this.skills.get(type);
	}

	public void processXPPacket(XPPacket xp) {
		for (SkillType skillType : SkillType.values()) {
			Skill skill = this.skills.get(skillType);
			skill.processXPPacket(xp);
			
			// automatically attempt to distribute points if ability was raised
			this.investTo(skillType);
		}
	}
	
	public void investTo(SkillType type) {
		
		Skill skill = this.getSkill(type);
		
		if (skill.getCombatValue() < skill.getMaxValue() && this.getSkillPoints() > 0) {
			skill.setCombatValue(skill.getCombatValue()+1);
			this.skillPoints--;
		}
		
	}
	
	public void uninvestTo(SkillType type) {
		
		Skill skill = this.getSkill(type);
		
		if (skill.getCombatValue() > 0) {
			skill.setCombatValue(skill.getCombatValue()-1);
			this.skillPoints++;
		}
		
	}

}

