package foh.rpg.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.BufferedImage;

import javax.swing.JComponent;

public class Console extends JComponent {
	private static final long serialVersionUID = 1L;

	private int screenResolutionX = 40;
	private int screenResolutionY = 24;
	
	private Representation[][] screen;
	
	private int cursorX = 1;
	private int cursorY = 1;
	
    public Console() {
		this.screen = new Representation[this.screenResolutionX][this.screenResolutionY];
		
		this.cursorX = 1;
		this.cursorY = 1;
		
    	this.cls();
    	
    	this.setSize(this.getPreferredSize());
    	
    }
    
    public void cls() {
    	for (int iWidth = 0; iWidth < this.screenResolutionX; iWidth++) {
			for (int iHeight = 0; iHeight < this.screenResolutionY; iHeight++) {
				this.screen[iWidth][iHeight] =  Representation.getEmpty();
			}
		}
    }
    
    public Dimension getPreferredSize() {
    	return new Dimension(TileFactory.getInstance().getTileWidth() * this.screenResolutionX, TileFactory.getInstance().getTileHeight() * this.screenResolutionY);
    }
    
    public Dimension getMinimumSize() {
    	return this.getPreferredSize();
    }
    
    public int getCursorX() {
    	return this.cursorX;
    }
    
    public int getCursorY() {
    	return this.cursorY;
    }

    protected void paintComponent(Graphics g) {

        Graphics2D g2d = (Graphics2D)g.create();
        
        BufferedImage img = new BufferedImage(this.getWidth(),this.getHeight(), BufferedImage.TYPE_INT_ARGB); 
        
        Graphics2D g2dBuffer = img.createGraphics();
        
        g2dBuffer.setBackground(Color.BLACK);
        g2dBuffer.clearRect(0,0,this.getWidth(),this.getHeight());
        g2dBuffer.setColor(Color.LIGHT_GRAY);
        
    	for (int x = 0; x < this.screenResolutionX; x++) {
			for (int y = 0; y < this.screenResolutionY; y++) {
				Image tileImage = TileFactory.getInstance().getTile(this.screen[x][y].getModelKey(), this.screen[x][y].getModelColor(), this.screen[x][y].getBackgroundColor()).getGraphics();
				g2dBuffer.drawImage(tileImage, x * TileFactory.getInstance().getTileWidth(), y * TileFactory.getInstance().getTileHeight(), null);
			}
		}

        g2d.drawImage(img, 0, 0, null);
        
        g2d.dispose();
        g2dBuffer.dispose();
        
	}
    
    public void moveCursorTo(int x, int y) {
    	this.cursorX = x;
    	this.cursorY = y;
    }
    
    public void printTextAt(String text, int x, int y) {
    	// first, lets convert text to char array
    	char[] textArray = text.toCharArray();
    	
    	int actualX = x;
    	int actualY = y;
    	
    	for (int i = 0; i < textArray.length; i++) {
    		
    		this.screen[actualX-1][actualY-1] = new Representation(""+textArray[i], Color.LIGHT_GRAY);
    		
			//printAt(textArray[i], actualX, actualY, g2d);
			
			// after printing, we have to update location for next cahr
			if (actualX >= this.screenResolutionX) {
				// we wrap around and go to next line if we reach end of line
				actualX = 1;
				if (actualY >= this.screenResolutionY) {
					// also, we wrap if we reach end of screen
					actualY = 1;				
				} else {
					actualY++;
				}
			} else {
				actualX++;
			}
				
		}

    	moveCursorTo(actualX, actualY);
    	
    	this.repaint();
    	
    }
   
    public void printText(String text) {
    	this.printTextAt(text, this.cursorX, this.cursorY);
    }
    
    public void printTextLn(String text) {
    	this.printTextAt(text, this.cursorX, this.cursorY);
    	this.cursorX = 1;
    	this.cursorY++;
    	if (this.cursorY >= this.screenResolutionY) {
    		this.cursorY = 1;				
		} 
    }

	/**
	 * @return Returns the screen.
	 */
	public Representation[][] getScreen() {
		return this.screen;
	}

	public void printText(char keyChar) {
		
		char[] chars = new char[1];
		chars[0] = keyChar;
		String x = new String(chars);
		
		this.printText( x );
		
	}

	/**
	 * @param screen The screen to set.
	 */
	public void setScreen(Representation[][] screen) {
		this.screen = screen;
	}

	/**
	 * @return Returns the screenResolutionX.
	 */
	public int getScreenResolutionX() {
		return this.screenResolutionX;
	}

	/**
	 * @return Returns the screenResolutionY.
	 */
	public int getScreenResolutionY() {
		return this.screenResolutionY;
	}

	public void setScreenColorXY(int fromX, int fromY, int toX, int toY, Color color) {
		
		for (int x = fromX; x < toX; x++) {
			for (int y = fromY; y < toY; y++) {
				this.screen[x][y].setModelColor(color);
			}
		}
	}
	
	public void setScreenColorXY(int x, int y, Color color) {
		
		this.screen[x][y].setModelColor(color);

	}

	public void setScreenBackgroundColorXY(int x, int y, Color color) {
		this.screen[x][y].setBackgroundColor(color);
		
	}

}