package foh.rpg.gui;

import java.awt.Color;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.List;
import java.util.concurrent.LinkedBlockingQueue;

import javax.swing.JFrame;

import foh.rpg.area.Cell;
import foh.rpg.character.Target;
import foh.rpg.model.Model;

public class Gui {
	
	static JFrame frame = null;
	static Console console = null;
	
	static boolean keysEcho = false;
	
	static LinkedBlockingQueue<KeyEvent> keyboardInput = new LinkedBlockingQueue<KeyEvent>();
	
	private Gui() {
		super();
	}
	
    public static void createAndShowGUI() {
        //Make sure we have nice window decorations.
    	
    	keysEcho = false;
    	
        JFrame.setDefaultLookAndFeelDecorated(true);

        //Create and set up the window.
        frame = new JFrame("Fate Of Heroes");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        
        console = new Console();
        
        frame.setFocusTraversalKeysEnabled(false);
        
        frame.getContentPane().add(console);
        
        frame.getContentPane().setSize(console.getPreferredSize());
        
        frame.setResizable(false);
        
        frame.addKeyListener(new KeyListener() {
        	public void keyTyped(KeyEvent e) {
        		//
			}

			public void keyPressed(KeyEvent e) {
				keyboardInput.add(e);
				if (keysEcho) {
					console.printText(e.getKeyChar());
				}
			}

			public void keyReleased(KeyEvent e) {
				//
			}
		}

        
        );
        

        //Display the window.
        frame.pack();
        frame.setVisible(true);
        
        frame.doLayout();
        
    }
    
	/**
	 * @return Returns the console.
	 */
	public static Console getConsole() {
		return console;
	}
	
	public static boolean isInput() {
		return !keyboardInput.isEmpty();
	}
	
	
	public static KeyEvent getNextKey() {
		
		KeyEvent key = null;
		try {
			key = keyboardInput.take();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		return key;
	}

	public static String getStringKey() {
		
		StringBuffer buffer = new StringBuffer();
		
		keysEcho = true;
		
		char key = getNextKey().getKeyChar();
		
		while ( key != 10 ) {
			buffer.append(key);
			key = getNextKey().getKeyChar();
		}
		
		keysEcho = false;

		return buffer.toString();
	}

	public static void displayModelMessages(Model model) {

		while (!model.getMessageBuffer().isEmpty()) {

			Gui.getConsole().moveCursorTo(1, 1);
			StringBuilder builder = new StringBuilder();
			
			while (!model.getMessageBuffer().isEmpty() && model.getMessageBuffer().getFirst().length() + builder.length() < 300) {
				builder.append(model.getMessageBuffer().getFirst());
				builder.append(" ");
				model.getMessageBuffer().removeFirst();
			}
			
			Gui.getConsole().printTextLn(builder.toString());
			if (!model.getMessageBuffer().isEmpty()) {
				Gui.getConsole().printText(" [MORE]");
				do {
					Thread.yield();
				} while (Gui.getNextKey().getKeyCode() != KeyEvent.VK_SPACE);
				Gui.getConsole().setScreen(
						model.buildScreen(Gui.getConsole()
								.getScreenResolutionX(), Gui
								.getConsole().getScreenResolutionY()));
			}

		}
		
		
	}

	public static void refreshModelDisplay(Model model) {
		
		Gui.getConsole().setScreen(
				model.buildScreen(Gui.getConsole()
						.getScreenResolutionX(), Gui.getConsole()
						.getScreenResolutionY()));
		
		// hp, mp
		
		Gui.getConsole().printTextAt("HP: " + model.getPc().getHP() + " MP: " + model.getPc().getMP(), 1, Gui.getConsole().getScreenResolutionY()-2);
		
		// ability
		
		if (model.getReadyAbility() != null) {
			Gui.getConsole().printTextAt("S: {" + model.getReadyAbility().getName() + "}", 1, Gui.getConsole().getScreenResolutionY()-1);
		} else {
			Gui.getConsole().printTextAt("S: {none}", 1, Gui.getConsole().getScreenResolutionY()-1);
		}
		
		// target
		
		if (model.getSelectedTarget() != null) {
			Gui.getConsole().printTextAt("T: {" + model.getSelectedTarget().getName() + ":" + model.getSelectedTarget().getHP() + "hp}", 25, Gui.getConsole().getScreenResolutionY()-1);
			
			// also, color tile underneath it
			
			Cell tile = model.getSelectedTarget().getCurrentArea().getActorTile(model.getSelectedTarget());
			
			int centerX = (Gui.getConsole()
					.getScreenResolutionX() - 1) / 2;
			int centerY = (Gui.getConsole()
					.getScreenResolutionY() - 1) / 2;
			
			int cursorScreenPositionX = tile.getPosition().getX() + (centerX - model.getPCPosition().getX());
			int cursorScreenPositionY = tile.getPosition().getY() + (centerY - model.getPCPosition().getY());
			
			if(cursorScreenPositionY >= 0 && cursorScreenPositionX >= 0 && cursorScreenPositionY < Gui.getConsole()
					.getScreenResolutionY() && cursorScreenPositionX < Gui.getConsole().getScreenResolutionX() ) {
				Gui.getConsole().setScreenBackgroundColorXY(cursorScreenPositionX, cursorScreenPositionY, Color.GRAY);
			}
			
		} else {
			Gui.getConsole().printTextAt("T: {none}", 25, Gui.getConsole().getScreenResolutionY()-1);
		}
		
		// ap display
		
		for (int i = 0; i < model.getPc().getTotalActionPoints(); i++) {
			if (i <= model.getPc().getActionPoints()) {
				Gui.getConsole().printTextAt("(*", i*2+1, Gui.getConsole().getScreenResolutionY());
				Gui.getConsole().setScreenColorXY(i*2+1, Gui.getConsole().getScreenResolutionY()-1, Color.GREEN);
			} else {
				Gui.getConsole().printTextAt("( ", i*2+1, Gui.getConsole().getScreenResolutionY());
			}
		}
		
		Gui.getConsole().repaint();
		
		if (model.inCombat()) {
			SoundSystem.getInstance().combatMusic();
		} else {
			SoundSystem.getInstance().exploreMusic();
		}
		
		
	}
	
	public static MenuItem getInlineMenuItem(List<MenuItem> menuModel, MenuItem escapeItem, int x, int y) {
		
		boolean selected = false;
		int selectedItemIndex = 0;
		
		while (!selected) {
			
			Gui.getConsole().moveCursorTo(x, y);
			
			for (int i = 0; i < menuModel.size(); i++) {
				String item = "[" + menuModel.get(i).getKey() + "] ";
				item += menuModel.get(i).getLabel();
				
				if ((Gui.getConsole().getScreenResolutionX() - Gui.getConsole().getCursorX()) < item.length() ) {
					Gui.getConsole().printTextLn("");
				}
				
				Gui.getConsole().printText(item);
				
				if (i == selectedItemIndex) {
						Gui.getConsole().setScreenColorXY(Gui.getConsole().getCursorX() - item.length() - 1, Gui.getConsole().getCursorY()-1, Gui.getConsole().getCursorX(), Gui.getConsole().getCursorY(), Color.YELLOW );
				}
				
				Gui.getConsole().setScreenColorXY(Gui.getConsole().getCursorX() - item.length(), Gui.getConsole().getCursorY()-1, Color.GREEN );
				Gui.getConsole().printText(" ");
			}
			
			
			KeyEvent key = Gui.getNextKey();
			
			if (key.getKeyCode() == KeyEvent.VK_LEFT || key.getKeyCode() == KeyEvent.VK_UP) {
				SoundSystem.getInstance().bloop();
				if (selectedItemIndex > 0) {
					selectedItemIndex--;
				}
			} else if (key.getKeyCode() == KeyEvent.VK_RIGHT || key.getKeyCode() == KeyEvent.VK_DOWN) {
				SoundSystem.getInstance().bleep();
				if (selectedItemIndex < menuModel.size()-1) {
					selectedItemIndex++;
				}
			} else if (key.getKeyCode() == KeyEvent.VK_ENTER) {
				SoundSystem.getInstance().select();
				return menuModel.get(selectedItemIndex);
			} else if (key.getKeyCode() == KeyEvent.VK_ESCAPE) {
				SoundSystem.getInstance().cancel();
				return escapeItem;
			} else {
				// check if one if menu items has this code
				
				for (MenuItem menuItem : menuModel) {
					if (menuItem.getKey() == key.getKeyCode()) {
						if (escapeItem == menuItem) {
							SoundSystem.getInstance().cancel();
						} else {
							SoundSystem.getInstance().select();
						}
						return menuItem;
					}
				}
				
			}
			
			
		}
		
		return menuModel.get(selectedItemIndex);
		
	}

	public static Integer getMenuItemIndex(List<MenuItem> menuModel, String title) {

		
		int height = console.getScreenResolutionY()-3;
		int fromItem = 0;
		int selectedItemIndex = 0;
		int toItem = (height > menuModel.size()) ? menuModel.size() : height+1;
		boolean selected = false;
		//
		
		while (!selected) {
			
			
			// repaint

			Gui.getConsole().cls();
			
			Gui.getConsole().moveCursorTo(1, 1);
			Gui.getConsole().printText(title);
			
			for (int i = fromItem ; i < toItem; i++) {
				Gui.getConsole().moveCursorTo(1, (i+2)-fromItem);
				char keyChar = (char)((int)'a' + (i - fromItem));
				
				String item = "[" + keyChar + "] ";
				if (menuModel.get(i).getIcon() != null) {
					item += "  ";
				}
				item += menuModel.get(i).getLabel();
				
				Gui.getConsole().printText(item);
				
				if (i == selectedItemIndex) {
					console.setScreenColorXY(0, (i+1) - fromItem, console.getScreenResolutionX(), (i+2) - fromItem, Color.YELLOW);
				}
				
				console.setScreenColorXY(1, (i+1) - fromItem, Color.GREEN);
				
				if (menuModel.get(i).getIcon() != null) {
					Gui.getConsole().getScreen()[4][Gui.getConsole().getCursorY()-1] = menuModel.get(i).getIcon(); 
				}
				
			}

			Gui.getConsole().moveCursorTo(Gui.getConsole().getScreenResolutionX(), 1);
			Gui.getConsole().printText((char)30);
			if (fromItem > 0) {
				console.setScreenColorXY(Gui.getConsole().getScreenResolutionX()-1, 0, Color.GREEN);
			} else {
				console.setScreenColorXY(Gui.getConsole().getScreenResolutionX()-1, 0, Color.DARK_GRAY);
			}
			Gui.getConsole().moveCursorTo(Gui.getConsole().getScreenResolutionX(), Gui.getConsole().getScreenResolutionY());
			Gui.getConsole().printText((char)31);
			if (toItem < menuModel.size()) {
				console.setScreenColorXY(Gui.getConsole().getScreenResolutionX()-1, Gui.getConsole().getScreenResolutionY()-1, Color.GREEN);
			} else {
				console.setScreenColorXY(Gui.getConsole().getScreenResolutionX()-1, Gui.getConsole().getScreenResolutionY()-1, Color.DARK_GRAY);
			}
			
			Gui.getConsole().moveCursorTo(1, Gui.getConsole().getScreenResolutionY());
			Gui.getConsole().printText("[Enter] to select, [Esc] to cancel.");
			
			console.setScreenColorXY(1, Gui.getConsole().getScreenResolutionY()-1, 6, Gui.getConsole().getScreenResolutionY(), Color.GREEN);
			console.setScreenColorXY(20, Gui.getConsole().getScreenResolutionY()-1, 23, Gui.getConsole().getScreenResolutionY(), Color.GREEN);
			
			KeyEvent key = Gui.getNextKey();
			
			if (key.getKeyCode() == KeyEvent.VK_UP || key.getKeyCode() == KeyEvent.VK_PAGE_UP || key.getKeyCode() == KeyEvent.VK_HOME ) {
				
				SoundSystem.getInstance().bloop();
				
				if (key.getKeyCode() == KeyEvent.VK_PAGE_UP) {
					selectedItemIndex-= height;
				} else if (key.getKeyCode() == KeyEvent.VK_HOME) {
					selectedItemIndex = 0;
				} else {
					selectedItemIndex--;
				}
				
				if (selectedItemIndex < 0) {
					selectedItemIndex = 0;
				}
				
				if (selectedItemIndex < fromItem) {
					fromItem = selectedItemIndex;
					
					toItem = fromItem+height+1;
					if (toItem > menuModel.size()) {
						toItem = menuModel.size();
					}
				}
				
			} else if (key.getKeyCode() == KeyEvent.VK_DOWN || key.getKeyCode() == KeyEvent.VK_PAGE_DOWN || key.getKeyCode() == KeyEvent.VK_END) {
				
				SoundSystem.getInstance().bleep();
				
				if (key.getKeyCode() == KeyEvent.VK_PAGE_DOWN) {
					selectedItemIndex+=height; 
				} else if (key.getKeyCode() == KeyEvent.VK_END) {
					selectedItemIndex = menuModel.size()-1;
				} else {
					selectedItemIndex++;
				}
				
				if (selectedItemIndex > menuModel.size()-1) {
					selectedItemIndex = menuModel.size()-1;
				}
				
				fromItem = selectedItemIndex-height;
				if (fromItem < 0) {
					fromItem = 0;
				}
				
				toItem = fromItem+height+1;
				if (toItem > menuModel.size()) {
					toItem = menuModel.size();
				}
					
			} else if (key.getKeyCode() == KeyEvent.VK_ENTER) {
				
				SoundSystem.getInstance().select();
				return selectedItemIndex;
				
			} else if (key.getKeyCode() == KeyEvent.VK_ESCAPE){
				
				SoundSystem.getInstance().cancel();
				return null;
				
			} else  {
				// direct key select
				
				int index = ((int)key.getKeyCode() - (int)'A') + fromItem;
				
				if (index > toItem) {
					index = toItem; 
				} else if(index < fromItem) {
					index = fromItem;
				}
				
				selectedItemIndex = index;
				
			}
			
		}
		
		return selectedItemIndex;
	}
	
	public static void clearScreen() {
		Gui.getConsole().cls();
		Gui.getConsole().moveCursorTo(1, 1);
	}

	public static Cell targetTile(Model model, Target target) {
		
		int targetX = model.getPCPosition().getX();
		int targetY = model.getPCPosition().getY();
		
		int centerX = (Gui.getConsole()
				.getScreenResolutionX() - 1) / 2;
		int centerY = (Gui.getConsole()
				.getScreenResolutionY() - 1) / 2;
		
		while (true) {
			
			refreshModelDisplay(model);
			
			int cursorScreenPositionX = targetX + (centerX - model.getPCPosition().getX());
			int cursorScreenPositionY = targetY + (centerY - model.getPCPosition().getY());
			
			Gui.getConsole().setScreenBackgroundColorXY(cursorScreenPositionX, cursorScreenPositionY, Color.YELLOW);
			
			Cell targetTile = model.getArea().getTile()[targetX][targetY];

			Gui.getConsole().printTextAt("[ESC] to leave, [ENTER] to select", 1, 1);
			Gui.getConsole().setScreenColorXY(1, 0, 4, 1, Color.GREEN);
			
			if (targetTile.isVisible() && target == Target.TILE) {
				Gui.getConsole().setScreenColorXY(17, 0, 22, 1, Color.GREEN);
			} else if (targetTile.isVisible() && target == Target.ACTOR && targetTile.getActor() != null) {
				Gui.getConsole().setScreenColorXY(17, 0, 22, 1, Color.GREEN);
			} else {
				Gui.getConsole().setScreenColorXY(17, 0, 22, 1, Color.RED);
			}
			
			StringBuffer tileDescription = new StringBuffer();
			
			tileDescription.append(targetTile.getDescription());
			
			if (targetTile.getItems().size() > 1) {
				tileDescription.append(" with pile of items.");
			} else if (targetTile.getItems().size() == 1) {
				tileDescription.append(". A ");
				tileDescription.append(targetTile.getItems().get(0).getName());
				tileDescription.append(".");
			}
			
			if (targetTile.getActor() != null) {
				tileDescription.append(" A ");
				tileDescription.append(targetTile.getActor().getName());
				tileDescription.append(".");
			}
			
			if (targetTile.isVisible()) {
				Gui.getConsole().printTextAt(tileDescription.toString(), 1, 2);
			} else if (targetTile.isKnown()) {
				Gui.getConsole().printTextAt(targetTile.getDescription(), 1, 2);
			} else {
				Gui.getConsole().printTextAt("Unknown location.", 1, 2);
			}
			
			KeyEvent key;
			if (Gui.isInput()) {
				key = Gui.getNextKey();
			} else {
				try {
					Thread.sleep(100);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
				key = null;
			}
			
			if (key != null) {
				switch (key.getKeyCode()) {
					case KeyEvent.VK_UP:
						if (targetY > 0 && cursorScreenPositionY > 0) {
							targetY--;
						}
						break;
					case KeyEvent.VK_DOWN:
						if (targetY < model.getArea().getHeight() - 1 && cursorScreenPositionY < Gui.getConsole().getScreenResolutionY() - 1) {
							targetY++;
						}
						break;
					case KeyEvent.VK_LEFT:
						if (targetX > 0 && cursorScreenPositionX > 0) {
							targetX--;
						}
						break;
					case KeyEvent.VK_RIGHT:
						if (targetX < model.getArea().getWidth() - 1 && cursorScreenPositionX < Gui.getConsole().getScreenResolutionX() - 1) {
							targetX++;
						}
						break;
					case KeyEvent.VK_ENTER:
						if (targetTile.isVisible() && target == Target.TILE) {
							return targetTile;
						} else if (targetTile.isVisible() && target == Target.ACTOR && targetTile.getActor() != null) {
							return targetTile;
						}
						break;
					case KeyEvent.VK_ESCAPE:
						return null;
				}
			}
			
		}
		
		
	}
	
}
