package foh.rpg.gui;

import javax.sound.midi.Instrument;
import javax.sound.midi.InvalidMidiDataException;
import javax.sound.midi.MidiChannel;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.MidiUnavailableException;
import javax.sound.midi.Receiver;
import javax.sound.midi.ShortMessage;
import javax.sound.midi.Synthesizer;

import foh.rpg.gui.music.MusicThread;

public class SoundSystem {
	
	private static SoundSystem instance;
	
	// Midi effects.
	private Synthesizer synth;
	private MidiChannel[] midiChannels;
	private int currentMidiChannel;
	
	// mp3
	private MusicThread musicThread;
	
	private SoundSystem() {
		try {
			this.synth = MidiSystem.getSynthesizer();
			this.synth.open();
			
			this.midiChannels = synth.getChannels();
			this.currentMidiChannel = 0;
			
			Instrument[] intruments = synth.getAvailableInstruments();
			for (int i = 0; i < intruments.length; i++) {
				synth.loadInstrument(intruments[i]);
			}
			
			this.musicThread = new MusicThread();
			
			new Thread(musicThread).start();
			
		} catch (MidiUnavailableException e) {
			e.printStackTrace();
		}

	}
	
	/**
	 * Rotates to next midi channel
	 */
	private int getNextMidiChannel() {
		if (this.currentMidiChannel < (this.midiChannels.length-1)) {
			this.currentMidiChannel++;
		} else {
			this.currentMidiChannel=0;
		}
 		return this.currentMidiChannel;
	}
	
	public static SoundSystem getInstance() {
		if (instance == null) {
			instance = new SoundSystem();
		}
		return instance;
	}
	
	public void bloop() {
	    ShortMessage myMsg = new ShortMessage();
	    try {
			myMsg.setMessage(ShortMessage.NOTE_ON, getNextMidiChannel(), 40, 93);
			synth.getChannels()[this.currentMidiChannel].programChange(115);
			Receiver synthRcvr = this.synth.getReceiver();
			synthRcvr.send(myMsg, -1); // -1 means no time stamp
		} catch (InvalidMidiDataException e) {
			e.printStackTrace();
		} catch (MidiUnavailableException e) {
			e.printStackTrace();
		}
	}
	
	public void bleep() {
	    ShortMessage myMsg = new ShortMessage();
	    try {
			myMsg.setMessage(ShortMessage.NOTE_ON, getNextMidiChannel(), 50, 93);
			synth.getChannels()[this.currentMidiChannel].programChange(115);
			Receiver synthRcvr = this.synth.getReceiver();
			synthRcvr.send(myMsg, -1); // -1 means no time stamp
		} catch (InvalidMidiDataException e) {
			e.printStackTrace();
		} catch (MidiUnavailableException e) {
			e.printStackTrace();
		}
	}

	public void select() {
	    ShortMessage myMsg = new ShortMessage();
	    try {
			myMsg.setMessage(ShortMessage.NOTE_ON, getNextMidiChannel(), 50, 93);
			synth.getChannels()[this.currentMidiChannel].programChange(113);
			Receiver synthRcvr = this.synth.getReceiver();
			synthRcvr.send(myMsg, -1); // -1 means no time stamp
		} catch (InvalidMidiDataException e) {
			e.printStackTrace();
		} catch (MidiUnavailableException e) {
			e.printStackTrace();
		}
	}
	
	public void cancel() {
	    ShortMessage myMsg = new ShortMessage();
	    try {
			myMsg.setMessage(ShortMessage.NOTE_ON, getNextMidiChannel(), 50, 93);
			synth.getChannels()[this.currentMidiChannel].programChange(120);
			Receiver synthRcvr = this.synth.getReceiver();
			synthRcvr.send(myMsg, -1); // -1 means no time stamp
		} catch (InvalidMidiDataException e) {
			e.printStackTrace();
		} catch (MidiUnavailableException e) {
			e.printStackTrace();
		}
	}
	
	public void menuMusic() {
		this.musicThread.setRequest("Achilles.mp3");
	}
	
	public void combatMusic() {
		this.musicThread.setRequest("Truth of the Legend.mp3");
	}
	
	public void exploreMusic() {
		this.musicThread.setRequest("Long Road Ahead.mp3");
	}

}
