package foh.rpg.gui;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.awt.image.FilteredImageSource;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.imageio.ImageIO;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import foh.rpg.util.FactoryFromXML;

public class TileFactory extends FactoryFromXML {
	
	private static TileFactory instance;

	private BufferedImage asciiImage = null;
	private Map<String, Tile> cache = new HashMap<String, Tile>();
	
	private int tileWidth = 0;
	private int tileHeight = 0;
	

	private TileFactory() {
		Document doc = this.parseDocument("tileset.xml");
		
		Node resolution = doc.getElementsByTagName("resolution").item(0);
		String tileWidthText = resolution.getAttributes().getNamedItem("tileWidth").getNodeValue();
		this.tileWidth = Integer.parseInt(tileWidthText);
		String tileHeightText = resolution.getAttributes().getNamedItem("tileHeight").getNodeValue();
		this.tileHeight = Integer.parseInt(tileHeightText);
		
		Node ascii = doc.getElementsByTagName("ascii").item(0);
		String asciiImageName = ascii.getAttributes().getNamedItem("path").getNodeValue();
		
		try {
			this.asciiImage = ImageIO.read(this.getClass().getClassLoader().getResource(asciiImageName));
		} catch (IOException e) {
			e.printStackTrace();
		}
			
	}

	public static TileFactory getInstance() {
		if (TileFactory.instance == null) {
			TileFactory.instance = new TileFactory();
		}
		return TileFactory.instance;
	}
	
	public Tile getTile(String key, Color color, Color bgColor) {
		
		String trueKey = key + "_" + color + "_" + bgColor;
		
		if (this.cache.containsKey(trueKey)) {
			return this.cache.get(trueKey);
		} else {
			
			Tile tile = new Tile();
			
			boolean asciiTile = false;
			int character = 0; 
			
			if (key.length() == 1) {
				asciiTile = true;
				character = key.charAt(0);
			} else {
				
				try {
					character = Integer.parseInt(key);
					asciiTile = true;
				} catch (NumberFormatException e) {
					asciiTile = false;
				}
				
			}
			
			if (asciiTile) {
				// simple tile, we will use mayday set
				
				int tilesetX = (int)character % this.getTileWidth();
				int tilesetY = (int)character / this.getTileHeight();
				
				BufferedImage img = new BufferedImage(this.getTileWidth(), this.getTileHeight(), BufferedImage.TYPE_INT_ARGB);  
				  
	            Graphics2D g = img.createGraphics();
	            
	            if (bgColor == null) {
	            	bgColor = Color.BLACK;
	            }
	            
				g.drawImage(asciiImage,
						0, 0, 
						this.getTileWidth(), this.getTileHeight(),
						tilesetX * this.getTileWidth(), tilesetY * this.getTileHeight(),
						tilesetX * this.getTileWidth() + this.getTileWidth(), tilesetY * this.getTileHeight() + this.getTileHeight(), null);  
				g.dispose();
				
				FilteredImageSource filter = new FilteredImageSource(img.getSource(), new TileImageFilter(color, bgColor));
				
				tile.setGraphics(Toolkit.getDefaultToolkit().createImage(filter));
				
			}
			
			this.cache.put(trueKey, tile);
			return tile;
		
		}
		
		
	}

	public int getTileWidth() {
		return this.tileWidth;
	}
	
	public int getTileHeight() {
		return this.tileHeight;
	}
	
}
