package foh.rpg.item;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import foh.rpg.character.Ability;
import foh.rpg.character.Actor;
import foh.rpg.character.SlotType;
import foh.rpg.character.attribute.AttributeType;
import foh.rpg.character.damage.DamageChunk;
import foh.rpg.character.damage.DamagePacket;
import foh.rpg.character.damage.DamageType;
import foh.rpg.character.skill.SkillType;
import foh.rpg.character.xp.XPPacket;
import foh.rpg.gui.Representation;
import foh.rpg.item.interfaces.Blessable;
import foh.rpg.item.magical.AtomicProperty;
import foh.rpg.item.magical.BonusType;
import foh.rpg.item.magical.MagicalProperty;
import foh.rpg.util.Dice;

/**
 * @author Zweistein
 */
public class Item implements Blessable {

	private String name = "";
	private String description = "";
	private int weight = 0;
	private List<SlotType> allowedSlotTypes = new ArrayList<SlotType>();
	private List<ItemRoles> itemRoles = new ArrayList<ItemRoles>();
	private List<ItemSocket> sockets = new ArrayList<ItemSocket>();
	

	public Blessing blessing = null;
	private int minDamage = 0;
	private int maxDamage = 0;
	private int attackRating = 0;
	private int defence = 0;
	private int block = 0;

	private List<MagicalProperty> magicalProperties = new ArrayList<MagicalProperty>();
	private Representation graphics;
	private SkillType governingSkill;
	private Ability linkedAbility;
	private int governingSkillInnateLevel;
	
	public int getGoverningSkillInnateLevel() {
		return this.governingSkillInnateLevel;
	}

	public void setGoverningSkillInnateLevel(int governingSkillInnateLevel) {
		this.governingSkillInnateLevel = governingSkillInnateLevel;
	}

	public List<MagicalProperty> getAllMagicalProperties() {
		List<MagicalProperty> properties = new LinkedList<MagicalProperty>();
		
		properties.addAll(this.getMagicalProperties());
		
		if (this.getLinkedAbility() != null) {
			properties.add(this.getLinkedAbility().getMagicalProperty(governingSkillInnateLevel));
		}
		
		for (ItemSocket socket : this.getSockets()) {
			if (socket.getContent() != null) {
				properties.addAll(socket.getContent().getAllMagicalProperties());
			}
		}
		
		return properties;
		
	} 
	
	public Ability getLinkedAbility() {
		return this.linkedAbility;
	}

	public void setLinkedAbility(Ability linkedAbility) {
		this.linkedAbility = linkedAbility;
	}

	public List<ItemSocket> getSockets() {
		return this.sockets;
	}

	/**
	 * @return Returns the magicalProperties.
	 */
	public List<MagicalProperty> getMagicalProperties() {
		return this.magicalProperties;
	}
	/**
	 * @param magicalProperties The magicalProperties to set.
	 */
	public void setMagicalProperties(List<MagicalProperty> magicalProperties) {
		this.magicalProperties = magicalProperties;
	}
	/**
	 * @return Returns the name.
	 */
	public String getName() {
		// now, we get prefixes and suffixes
		StringBuilder stringBuilder = new StringBuilder();
		
		List<String> prefixes = new ArrayList<String>();
		List<String> suffixes = new ArrayList<String>();
		
		for (Iterator<MagicalProperty> it = this.getMagicalProperties().iterator(); it.hasNext();) {
			MagicalProperty magic = it.next();
			if (magic.isPrefix()) {
				prefixes.add(magic.getPrefix());
			}
			if (magic.isSuffix()) {
				suffixes.add(magic.getSuffix());
			}
		}
		
		if (prefixes.size() != 0) {
			stringBuilder.append(prefixes.get(0));
			stringBuilder.append(" ");
		}
		
		stringBuilder.append(this.getBlessing().toString()).append(" ");
		
		stringBuilder.append(this.name);
		
		if (suffixes.size() != 0) {
			stringBuilder.append(" ");
			stringBuilder.append(suffixes.get(0));
		}
		
		// upgrade slots belong to name too
		
		if (this.getSockets().size() != 0) {
			stringBuilder.append(" ");
		}
		
		for (ItemSocket socket: this.getSockets()) {
			if (socket.getContent() == null) {
				stringBuilder.append("( )");
			} else {
				stringBuilder.append("(*)");
			}
		}
		
		return stringBuilder.toString();
		
	}
	/**
	 * @param name The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}
	/**
	 * @return Returns the weight.
	 */
	public int getWeight() {
		return this.weight;
	}
	/**
	 * @param weight The weight to set.
	 */
	public void setWeight(int weight) {
		this.weight = weight;
	}
	/**
	 * 
	 */
	public Item() {
		super();
		this.blessing = new Blessing(BlessingType.NORMAL);
	}
	
	public String toString() {
		
		StringBuffer message = new StringBuffer("");
		
		message.append(this.getName() + "\n");
		
		if (this.getDefence() != 0) {
			message.append("Defence: " + this.getDefence() + "\n");
		}
		
		message.append("Damage: " + this.getDamageDice() + "\n");

		if (this.getAttackRating() != 0) {
			message.append("Attack Bonus: " + this.getAttackRating() + "\n");	
		}
		
		if (this.getMagicalProperties().size() != 0) {
			message.append("Magic:\n");
			for (Iterator<MagicalProperty> it = this.getMagicalProperties().iterator(); it.hasNext();) {
				MagicalProperty magic = it.next();
				message.append(magic.toString());
			}
		}
		
		message.append("Weight:" + this.getWeight());
		
		return message.toString();
	}

	/**
	 * @return Returns the description.
	 */
	public String getDescription() {
		return this.description;
	}
	/**
	 * @param description The description to set.
	 */
	public void setDescription(String description) {
		this.description = description;
	}
	/**
	 * @return Returns the blessing.
	 */
	public Blessing getBlessing() {
		return this.blessing;
	}
	/**
	 * @param blessing The blessing to set.
	 */
	public void setBlessing(Blessing blessing) {
		this.blessing = blessing;
	}
	/**
	 * @return Returns the allowedSlots.
	 */
	public List<SlotType> getAllowedSlotTypes() {
		return this.allowedSlotTypes;
	}
	public Dice getDamageDice() {
		if (this.minDamage == 0 && this.maxDamage == 0) {
			return new Dice(1, 1);
		} else {
			return new Dice(this.getMinDamage(), this.getMaxDamage());
		}
		
	}
	/**
	 * @return Returns the attackRating.
	 */
	public int getAttackRating() {
		int tempAttackRating = this.attackRating;
		
		if (this.attackRating != 0) {
			if ( this.getBlessing().getBlessedStatus() == BlessingType.DOOMED ) {
				tempAttackRating-=50;
			} else if ( this.getBlessing().getBlessedStatus() == BlessingType.CURSED ) {
				tempAttackRating-=5;
			} else if ( this.getBlessing().getBlessedStatus() == BlessingType.NORMAL) {
				//
			} else if ( this.getBlessing().getBlessedStatus() == BlessingType.NORMAL) {
				tempAttackRating+=5;
			} else if ( this.getBlessing().getBlessedStatus() == BlessingType.HOLY) {
				tempAttackRating+=10;
			}	
		}
		
		int tmpMagicAbsoluteBonus = 0;
		
		for (MagicalProperty magicalProperty: this.getAllMagicalProperties()) {
			for (AtomicProperty property : magicalProperty.getProperties()) {
				if (property.getType() == BonusType.ATTACK_RATING_BONUS) {
					tmpMagicAbsoluteBonus += property.getAmount();
				}
			}
		}
		
		tempAttackRating = tempAttackRating + tmpMagicAbsoluteBonus; 
		
		return tempAttackRating;
	}
	/**
	 * @param attackRating The attackRating to set.
	 */
	public void setAttackRating(int attackRating) {
		this.attackRating = attackRating;
	}
	
	/**
	 * @return Returns the defence.
	 */
	public int getDefence() {
		
		int tempDefence = this.defence;
		
		return tempDefence;
	}
	
	/**
	 * @param defence The defence to set.
	 */
	public void setDefence(int defence) {
		this.defence = defence;
	}
	/**
	 * @return Returns the maxDamage.
	 */
	public int getMaxDamage() {
		
		int tempMaxDamage = this.maxDamage;

		if ( this.getBlessing().getBlessedStatus() == BlessingType.DOOMED ) {
			tempMaxDamage-=2;
		} else if ( this.getBlessing().getBlessedStatus() == BlessingType.CURSED ) {
			tempMaxDamage--;
		} else if ( this.getBlessing().getBlessedStatus() == BlessingType.NORMAL) {
			//
		} else if ( this.getBlessing().getBlessedStatus() == BlessingType.NORMAL) {
			tempMaxDamage++;
		} else if ( this.getBlessing().getBlessedStatus() == BlessingType.HOLY) {
			tempMaxDamage+=2;
		}
		
		return tempMaxDamage;
	}
	/**
	 * @param maxDamage The maxDamage to set.
	 */
	public void setMaxDamage(int maxDamage) {
		this.maxDamage = maxDamage;
	}
	/**
	 * @return Returns the minDamage.
	 */
	public int getMinDamage() {
		
		int tempMinDamage = this.minDamage;
		
		if ( this.getBlessing().getBlessedStatus() == BlessingType.DOOMED ) {
			tempMinDamage-=2;
		} else if ( this.getBlessing().getBlessedStatus() == BlessingType.CURSED ) {
			tempMinDamage--;
		} else if ( this.getBlessing().getBlessedStatus() == BlessingType.NORMAL) {
			//
		} else if ( this.getBlessing().getBlessedStatus() == BlessingType.NORMAL) {
			tempMinDamage++;
		} else if ( this.getBlessing().getBlessedStatus() == BlessingType.HOLY) {
			tempMinDamage+=2;
		}
		
		return tempMinDamage;
	}
	/**
	 * @param minDamage The minDamage to set.
	 */
	public void setMinDamage(int minDamage) {
		this.minDamage = minDamage;
	}
	
	public void onHit(Actor user, Actor target) {
		
		for (MagicalProperty magicProperty: this.getAllMagicalProperties()) {
			magicProperty.onHit(user, target);
		}
		
	}
	
	public DamagePacket generateDamagePacket(Actor owner) {
		DamagePacket damage = new DamagePacket();
		
		DamageChunk chunk = new DamageChunk();
		chunk.setDamageAmount(new Integer(this.getDamageDice().roll()) + owner.getSkills().getSkill(this.governingSkill).getCombatValue() + owner.getAttributes().getAttribute(AttributeType.STRENGTH).getCombatValue());
		chunk.setDamageType(DamageType.PHYSICAL_DAMAGE);
		chunk.setOriginatingSkill(this.governingSkill);
		
		damage.getDamageChunks().add(chunk);
		
		for (MagicalProperty magicalProperty: this.getAllMagicalProperties()) {
			magicalProperty.onMeleeDamagePacketGeneration(damage, null, null);
		}
		
		return damage;
		
	}
	
	public List<ItemRoles> getItemRoles() {
		return this.itemRoles;
	}
	
	public void setItemRoles(List<ItemRoles> itemRoles) {
		this.itemRoles = itemRoles;
	}
	
	public SkillType getGoverningSkill() {
		return this.governingSkill;
	}
	
	public void setGoverningSkill(SkillType governingSkill) {
		this.governingSkill = governingSkill;
	}
	
	public void consume(Actor user) {

		if (this.governingSkill != null) {
			XPPacket xp = new XPPacket();
			xp.getSkillXP().put(this.governingSkill, 10);
			user.processXPPacket(xp);
		}
		
		for (MagicalProperty property : this.getAllMagicalProperties()) {
			property.onConsume(user);
		}
		
	}
	
	public Representation getGraphics() {
		return this.graphics;
	}
	
	public void setGraphics(Representation representation) {
		this.graphics = representation;
	}

	public int getBlock() {
		return this.block;
	}
	
	public void setBlock(int block) {
		this.block = block;
	}
	
}
