package foh.rpg.item.factory;

import java.awt.Color;
import java.util.HashMap;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import foh.rpg.character.Ability;
import foh.rpg.character.SlotType;
import foh.rpg.character.skill.SkillType;
import foh.rpg.gui.Representation;
import foh.rpg.item.Item;
import foh.rpg.item.ItemRoles;
import foh.rpg.item.ItemSocket;
import foh.rpg.item.magical.AtomicProperty;
import foh.rpg.item.magical.BonusType;
import foh.rpg.item.magical.MagicalProperty;
import foh.rpg.util.Dice;
import foh.rpg.util.FactoryFromXML;

/**
 * @author Zweistein
 */
public class ItemFactory extends FactoryFromXML {

	private static ItemFactory instance = null;
	
	private Map<String, ItemPrototype> itemPrototypes;

	public static ItemFactory getInstance() {
		if (instance == null) {
			instance = new ItemFactory();
		}
		return instance;
	}
	
	private ItemFactory() {
		this.itemPrototypes = new HashMap<String, ItemPrototype>();
		
		Document doc = this.parseDocument("items.xml");
		
		NodeList itemNodes = doc.getElementsByTagName("item");
		
		for (int i = 0; i < itemNodes.getLength(); i++) {
			Node itemNode = itemNodes.item(i);
			
			ItemPrototype prototype = new ItemPrototype();
			
			String prototypeKey = itemNode.getAttributes()
					.getNamedItem("id").getNodeValue();

			this.itemPrototypes.put(prototypeKey, prototype);
			
			prototype.setName(itemNode.getAttributes()
					.getNamedItem("name").getNodeValue());
			
			NodeList configurationElements = itemNode.getChildNodes();
			
			for (int j = 0; j < configurationElements.getLength(); j++) {
				Node configNode = configurationElements.item(j);
				String name = configNode.getLocalName();

				if ("representation".equals(name)) {
					
					String key = configNode.getAttributes().getNamedItem("key").getNodeValue();
					String color = configNode.getAttributes().getNamedItem("color").getNodeValue();
					
					prototype.setRepresentation(new Representation(key, colorParser.parse(color) ));
					
				} else if ("skill".equals(name)) {
					
					String skillType = configNode.getAttributes().getNamedItem("key").getNodeValue();
					prototype.setLinkedSkill(SkillType.valueOf(skillType));
					
				} else if ("damage".equals(name)) {
					
					String min = configNode.getAttributes().getNamedItem("min").getNodeValue();
					prototype.setMinDamage(diceParser.parse(min));
					String max = configNode.getAttributes().getNamedItem("max").getNodeValue();
					prototype.setMaxDamage(diceParser.parse(max));
					
				} else if ("defense".equals(name)) {
					
					String value = configNode.getAttributes().getNamedItem("value").getNodeValue();
					prototype.setDefense(diceParser.parse(value));
					
					
				} else if ("slots".equals(name)) {
					
					NodeList configSlotNodes = configNode.getChildNodes();
					for (int k = 0; k < configSlotNodes.getLength(); k++) {
						Node configSlotNode = configSlotNodes.item(k);
						String configSlotNodeName = configSlotNode.getLocalName();
						
						if ("slot".equals(configSlotNodeName)) {
							
							String nodeKey = configSlotNode.getAttributes().getNamedItem("key").getNodeValue();
							prototype.getAllowedSlotTypes().add(SlotType.valueOf(nodeKey));
							
						}
						
					}
					
				} else if ("roles".equals(name)) {
					
					NodeList configRoleNodes = configNode.getChildNodes();
					for (int k = 0; k < configRoleNodes.getLength(); k++) {
						Node configRoleNode = configRoleNodes.item(k);
						String configRoleNodeName = configRoleNode.getLocalName();
						
						if ("role".equals(configRoleNodeName)) {
							
							String nodeKey = configRoleNode.getAttributes().getNamedItem("key").getNodeValue();
							prototype.getItemRoles().add(ItemRoles.valueOf(nodeKey));
							
						}
						
					}
					
				} else if ("upgrades".equals(name)) {
					
					NodeList configUpgradeNodes = configNode.getChildNodes();
					for (int k = 0; k < configUpgradeNodes.getLength(); k++) {
						Node configUpgradeNode = configUpgradeNodes.item(k);
						String configUpgradeNodeName = configUpgradeNode.getLocalName();
						
						if ("runeSocket".equals(configUpgradeNodeName)) {
							
							String amount = configUpgradeNode.getAttributes().getNamedItem("amount").getNodeValue();
							prototype.setRuneSockets(diceParser.parse(amount));
							
						}
						
					}
					
				}
					
					
				
			}
			
		}
		
		
	}
	
	public Item getItem(String key) {
		if ("rune".equals(key)) {
			return this.generateRune();
		} else {
			return this.itemPrototypes.get(key).getItem();
		}
	}

	public Item generateItem() {

		Item test = new Item();

		switch ((new Dice(1, 4)).roll()) {
		case 1:

			MagicalProperty magic1 = new MagicalProperty();
			AtomicProperty property = new AtomicProperty();
			
			switch ((new Dice(1, 2)).roll()) {
			
			case 1:
			
				magic1.setPrefix("Surestrike");
				property.setAmount(5);
				property.setType(BonusType.ATTACK_RATING_BONUS);
				magic1.getProperties().add(property);
			
			break;
			
			case 2:
				
				magic1.setPrefix("Fiery");
				property.setAmount(5);
				property.setType(BonusType.FIRE_DAMAGE);
				magic1.getProperties().add(property);
			
			break;
			
			}

			test.setGraphics(new Representation("(", Color.GRAY));
			test.setName("Sword");
			test.getBlessing().bless();
			test.setMinDamage(1);
			test.setMaxDamage(6);
			test.getItemRoles().add(ItemRoles.WEAPON);
			test.setGoverningSkill(SkillType.SWORD_MASTERY);
			test.getAllowedSlotTypes().add(SlotType.HAND);

			test.getMagicalProperties().add(magic1);

			break;
		case 2:

			test.setGraphics(new Representation("^", Color.GRAY));
			test.setName("Cap");
			test.getBlessing().curse();
			test.setDefence(1);
			test.getItemRoles().add(ItemRoles.ARMOR);
			test.setGoverningSkill(SkillType.HEAVY_ARMOR_MASTERY);
			test.getAllowedSlotTypes().add(SlotType.HEAD_ARMOR);

			break;

		case 3:

			test.setGraphics(new Representation("]", Color.GRAY));
			test.setName("Shield");
			test.getBlessing().divinate();
			test.setDefence(2);
			test.setBlock(25);
			test.getItemRoles().add(ItemRoles.ARMOR);
			test.setGoverningSkill(SkillType.SHILED_MASTERY);
			test.getAllowedSlotTypes().add(SlotType.HAND);

			break;

		case 4:

			test.setGraphics(new Representation("]", Color.GRAY));
			test.setName("Breastplate");
			test.getBlessing().doom();
			test.setDefence(5);
			test.getItemRoles().add(ItemRoles.ARMOR);
			test.setGoverningSkill(SkillType.HEAVY_ARMOR_MASTERY);
			test.getAllowedSlotTypes().add(SlotType.BODY_ARMOR);

			break;
			
		}
		
		
		int socketNum = new Dice(0,2).roll();
		for (int i = 0; i < socketNum; i++) {
			ItemSocket socket = new ItemSocket();
			socket.getRoles().add(ItemRoles.RUNE);
			test.getSockets().add(socket);
		}

		return test;

	}

	public Item generateRune() {
		
		Item item = new Item();
		item.getItemRoles().add(ItemRoles.CONSUMBALE);
		item.getItemRoles().add(ItemRoles.UPGRADE);
		item.getItemRoles().add(ItemRoles.RUNE);
		item.getItemRoles().add(ItemRoles.ABILITY_CARRIER);
		item.setGoverningSkillInnateLevel(5);
		item.setLinkedAbility(Ability.values()[new Dice(0,Ability.values().length-1).roll()]);
		item.setName(item.getLinkedAbility().getName() + " rune");
		item.setGoverningSkill(item.getLinkedAbility().getLinkedSkill());
		item.setGraphics(new Representation("&", item.getLinkedAbility().getLinkedSkill().getColor()));
			
		return item;
	}
	
}
