package foh.rpg.item.factory;

import java.util.LinkedList;
import java.util.List;

import foh.rpg.character.SlotType;
import foh.rpg.character.skill.SkillType;
import foh.rpg.gui.Representation;
import foh.rpg.item.Item;
import foh.rpg.item.ItemRoles;
import foh.rpg.item.ItemSocket;
import foh.rpg.util.Dice;

public class ItemPrototype {
	
	private String name;
	private Representation representation;
	private List<SlotType> allowedSlotTypes;
	private List<ItemRoles> itemRoles;
	private SkillType linkedSkill;
	
	private Dice runeSockets;
	
	private Dice minDamage;
	private Dice maxDamage;
	private Dice defense;
	
	public ItemPrototype() {
		this.runeSockets = new Dice(0, 0);
		this.allowedSlotTypes = new LinkedList<SlotType>();
		this.itemRoles = new LinkedList<ItemRoles>();
	}
	
	public Item getItem() {
		
		Item item = new Item();
	
		item.setName(this.name);
		item.setGraphics(this.representation.clone());
		
		item.setGoverningSkill(this.linkedSkill);
		
		if (this.minDamage != null) {
			item.setMinDamage(this.minDamage.roll());
		}
		if (this.maxDamage != null) {
			item.setMaxDamage(this.maxDamage.roll());
		}
		if (this.defense != null) {
			item.setDefence(this.defense.roll());
		}
		
		int runeSocketAmount = runeSockets.roll();
		
		for (int i = 0; i < runeSocketAmount; i++) {
			ItemSocket socket = new ItemSocket();
			socket.getRoles().add(ItemRoles.RUNE);
			item.getSockets().add(socket);
		}
		
		for (SlotType slot: this.allowedSlotTypes) {
			item.getAllowedSlotTypes().add(slot);
		}
		
		for (ItemRoles itemRole: this.itemRoles) {
			item.getItemRoles().add(itemRole);
		}
		
		return item;
	}

	public Dice getDefense() {
		return this.defense;
	}

	public void setDefense(Dice defense) {
		this.defense = defense;
	}

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public Representation getRepresentation() {
		return this.representation;
	}

	public void setRepresentation(Representation representation) {
		this.representation = representation;
	}

	public List<SlotType> getAllowedSlotTypes() {
		return this.allowedSlotTypes;
	}

	public void setAllowedSlotTypes(List<SlotType> allowedSlotTypes) {
		this.allowedSlotTypes = allowedSlotTypes;
	}

	public List<ItemRoles> getItemRoles() {
		return this.itemRoles;
	}

	public void setItemRoles(List<ItemRoles> itemRoles) {
		this.itemRoles = itemRoles;
	}

	public SkillType getLinkedSkill() {
		return this.linkedSkill;
	}

	public void setLinkedSkill(SkillType linkedSkill) {
		this.linkedSkill = linkedSkill;
	}

	public Dice getRuneSockets() {
		return this.runeSockets;
	}

	public void setRuneSockets(Dice runeSockets) {
		this.runeSockets = runeSockets;
	}

	public Dice getMinDamage() {
		return this.minDamage;
	}

	public void setMinDamage(Dice minDamage) {
		this.minDamage = minDamage;
	}

	public Dice getMaxDamage() {
		return this.maxDamage;
	}

	public void setMaxDamage(Dice maxDamage) {
		this.maxDamage = maxDamage;
	}
	
}
