package foh.rpg.item.magical;

import java.awt.Color;
import java.util.List;

import foh.rpg.area.Cell;
import foh.rpg.character.Actor;
import foh.rpg.character.damage.DamageChunk;
import foh.rpg.character.damage.DamagePacket;
import foh.rpg.character.damage.DamageType;
import foh.rpg.character.effect.Effect;
import foh.rpg.gui.Representation;

public enum BonusType {
	
	LIGHTNING_DAMAGE {

		@Override
		public void onCast(AtomicProperty property, Actor user, Actor target) {
			
			DamageChunk chunk = new DamageChunk();
			chunk.setDamageAmount(property.getAmount());
			chunk.setDamageType(DamageType.LIGHTNING_DAMAGE);
			DamagePacket packet = new DamagePacket();
			packet.setOriginator(target);
			packet.getDamageChunks().add(chunk);
			target.processDamagePacket(packet);
			
		}

		@Override
		public void onMeleeDamagePacketGeneration(AtomicProperty property,
				DamagePacket packet, Actor source, Actor target) {
			DamageChunk chunk = new DamageChunk();
			chunk.setDamageAmount(property.getAmount());
			chunk.setDamageType(DamageType.LIGHTNING_DAMAGE);
			packet.getDamageChunks().add(chunk);
		}
		
	}, 
	
	ATTACK_RATING_BONUS,
	FIRE_DAMAGE {
		@Override
		public void onMeleeDamagePacketGeneration(AtomicProperty property, DamagePacket packet, Actor source, Actor target) {
			DamageChunk chunk = new DamageChunk();
			chunk.setDamageAmount(property.getAmount());
			chunk.setDamageType(DamageType.FIRE_DAMAGE);
			packet.getDamageChunks().add(chunk);
		}
		
		@Override
		public void onConsume(AtomicProperty property, Actor target) {
			// cause damage to adjancent foes.
			
			List<Cell> tiles = target.getCurrentArea().getActorTile(target).getAdjacentTiles();
			
			for (Cell cell : tiles) {
				if (cell.getActor() != null) {
					Actor adjacent = cell.getActor();
					DamageChunk chunk = new DamageChunk();
					chunk.setDamageAmount(property.getAmount());
					chunk.setDamageType(DamageType.FIRE_DAMAGE);
					DamagePacket packet = new DamagePacket();
					packet.setOriginator(target);
					packet.getDamageChunks().add(chunk);
					adjacent.processDamagePacket(packet);
					
				}
			}
		}
		
		@Override
		public void onCast(AtomicProperty property, Actor user, Actor target) {
			// cause damage to adjancent foes.
			
			List<Cell> tiles = user.getCurrentArea().getActorTile(user).getAdjacentTiles();
			
			for (Cell cell : tiles) {
				if (cell.getActor() != null) {
					Actor adjacent = cell.getActor();
					DamageChunk chunk = new DamageChunk();
					chunk.setDamageAmount(property.getAmount());
					chunk.setDamageType(DamageType.FIRE_DAMAGE);
					DamagePacket packet = new DamagePacket();
					packet.setOriginator(user);
					packet.getDamageChunks().add(chunk);
					adjacent.processDamagePacket(packet);
					
				}
			}
		}
	},
	HEALTH_RESTORE {
		@Override
		public void onConsume(AtomicProperty property, Actor user) {
			user.setHP(user.getHP()+property.getAmount());
		}
		
		@Override
		public void onHit(AtomicProperty property, Actor user, Actor target) {
			user.setHP(user.getHP()+property.getAmount());
		}
		
		@Override
		public void onCast(AtomicProperty property, Actor user, Actor target) {
			user.setHP(user.getHP()+property.getAmount());
		}
	},
	AOE_LIFESTEAL {
		
		@Override
		public void onHit(AtomicProperty property, Actor user, Actor target) {
			onCast(property, user, target);
		}
		
		@Override
		public void onConsume(AtomicProperty property, Actor user) {
			onCast(property, user, user);
		}
		
		@Override
		public void onCast(AtomicProperty property, Actor user, Actor target) {

			List<Cell> tiles = user.getCurrentArea().getActorTile(user).getAdjacentTiles();
			
			for (Cell cell : tiles) {
				if (cell.getActor() != null) {
					Actor adjacent = cell.getActor();
					DamageChunk chunk = new DamageChunk();
					chunk.setDamageAmount(property.getAmount());
					chunk.setDamageType(DamageType.LIFE_FORCE_DAMAGE);
					DamagePacket packet = new DamagePacket();
					packet.setOriginator(user);
					packet.getDamageChunks().add(chunk);
					adjacent.processDamagePacket(packet);
					user.setHP(user.getHP()+property.getAmount());
				}
			}
			
		}
		
	},
	AOE_ENERGYSTEAL {
		
		@Override
		public void onHit(AtomicProperty property, Actor user, Actor target) {
			onCast(property, user, target);
		}
		
		@Override
		public void onConsume(AtomicProperty property, Actor user) {
			onCast(property, user, user);
		}
		
		@Override
		public void onCast(AtomicProperty property, Actor user, Actor target) {

			List<Cell> tiles = user.getCurrentArea().getActorTile(user).getAdjacentTiles();
			
			for (Cell cell : tiles) {
				if (cell.getActor() != null) {
					Actor adjacent = cell.getActor();
					adjacent.setMP(user.getMP()-property.getAmount());
					user.setMP(user.getMP()+property.getAmount());
				}
			}
			
		}
		
	},
	
	BLESS {
		
		@Override
		public void onCast(AtomicProperty property, Actor user, Actor target) {
			
			Effect effect = new Effect();
			
			effect.setName("Bless");
			effect.setGraphics(new Representation("%", Color.WHITE));
			effect.getTimer().setExpiration(100);
			effect.getProperties().add(property);
			
			effect.setTarget(target);
			effect.setSource(user);
			
			target.getEffectStack().add(effect);
			
		}
		
		@Override
		public void onConsume(AtomicProperty property, Actor user) {
			this.onCast(property, user, user);
		}
		
		@Override
		public void onEffectMeleeDamagePacketGeneration(AtomicProperty property, DamagePacket packet, Actor source, Actor target) {
			DamageChunk chunk = new DamageChunk();
			chunk.setDamageAmount(property.getAmount());
			chunk.setDamageType(DamageType.HOLY_DAMAGE);
			packet.getDamageChunks().add(chunk);
		}
		
	},
	
	HEALTH_DRAIN {
		
		@Override
		public void onCast(AtomicProperty property, Actor user, Actor target) {
			Effect effect = new Effect();
			
			effect.setName("Wither");
			
			effect.setGraphics(new Representation("%", Color.DARK_GRAY));
			effect.getTimer().setExpiration(1000);
			effect.getProperties().add(property);
			
			effect.setTarget(target);
			effect.setSource(user);
			
			target.getEffectStack().add(effect);
		}
		
		@Override
		public void onConsume(AtomicProperty property, Actor user) {
			this.onCast(property, user, user);
		}
		
		@Override
		public void onEffectTick(AtomicProperty property, Actor source, Actor target) {
			DamagePacket damage = new DamagePacket();
			damage.setOriginator(source);
			DamageChunk chunk = new DamageChunk();
			chunk.setDamageAmount(property.getAmount());
			chunk.setDamageType(DamageType.SHADOW_DAMAGE);
			damage.getDamageChunks().add(chunk);
			target.processDamagePacket(damage);
		}
		
	},
	ICE_DAMAGE {
		
		@Override
		public void onCast(AtomicProperty property, Actor user, Cell target) {
			
			for (Cell cell: target.getAdjacentTiles()) {
				Effect effect = new Effect();
				effect.setName("Blizzard");
				effect.setGraphics(new Representation("%", Color.CYAN));
				effect.getTimer().setExpiration(50);
				effect.getProperties().add(property);
				effect.setTarget(null);
				effect.setSource(user);
				cell.getEffectStack().add(effect);
				cell.getEffectStack().setTarget(cell.getActor());
			}
			
			Effect effect = new Effect();
			effect.setName("Blizzard");
			effect.setGraphics(new Representation("%", Color.CYAN));
			effect.getTimer().setExpiration(1000);
			effect.getProperties().add(property);
			effect.setTarget(null);
			effect.setSource(user);
			target.getEffectStack().add(effect);
			target.getEffectStack().setTarget(target.getActor());
			
		}
		
		@Override
		public void onEffectTick(AtomicProperty property, Actor source, Actor target) {
			if (target != null) {
				DamagePacket damage = new DamagePacket();
				damage.setOriginator(source);
				DamageChunk chunk = new DamageChunk();
				chunk.setDamageAmount(property.getAmount());
				chunk.setDamageType(DamageType.ICE_DAMAGE);
				damage.getDamageChunks().add(chunk);
				target.processDamagePacket(damage);
			}
		}
		
	};

	public void onConsume(AtomicProperty property, Actor user) {
		// empty prototye.
	}
	
	public void onMeleeDamagePacketGeneration(AtomicProperty property, DamagePacket packet, Actor source, Actor target) {
		// empty prototype
	}
	
	public void onHit(AtomicProperty property, Actor user, Actor target) {
		// empty prototye.
	}

	public void onCast(AtomicProperty property, Actor user, Actor target) {
		// empty prototype
	}

	public void onEffectTick(AtomicProperty property, Actor source, Actor target) {
		// empty prototype
	}
	
	public void onEffectMeleeDamagePacketGeneration(AtomicProperty property, DamagePacket packet, Actor source, Actor target) {
		// empty prototype
	}

	public void onCast(AtomicProperty property, Actor user, Cell target) {
		// empty prototype
	}
	
}
