package foh.rpg.model;

import java.awt.Color;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import foh.rpg.area.Area;
import foh.rpg.area.AreaTools;
import foh.rpg.area.Cell;
import foh.rpg.area.factory.AreaFactory;
import foh.rpg.character.Ability;
import foh.rpg.character.AbilityCost;
import foh.rpg.character.Actor;
import foh.rpg.character.Slot;
import foh.rpg.character.Target;
import foh.rpg.character.attribute.Attribute;
import foh.rpg.character.attribute.AttributeType;
import foh.rpg.character.effect.Effect;
import foh.rpg.character.npc.dialogue.Conversation;
import foh.rpg.character.npc.dialogue.State;
import foh.rpg.character.npc.dialogue.Transition;
import foh.rpg.character.position.Position;
import foh.rpg.character.skill.Skill;
import foh.rpg.character.skill.SkillType;
import foh.rpg.character.xp.XPPacket;
import foh.rpg.gui.Gui;
import foh.rpg.gui.MenuItem;
import foh.rpg.gui.Representation;
import foh.rpg.gui.SoundSystem;
import foh.rpg.item.Item;
import foh.rpg.item.ItemRoles;
import foh.rpg.item.ItemSocket;
import foh.rpg.item.magical.MagicalProperty;

public class Model {

	private Actor pc = null;
	private Actor selectedTarget = null;
	private Position position = null;
	private Area area = null;
	private LinkedList<String> messageBuffer = new LinkedList<String>();
	private Ability readyAbility;
	public Model() {
		//
	}

	public void fireCommandEvent(int key) {
		switch (key) {
		case KeyEvent.VK_UP:
			this.movePc(this.getPCPosition().getX(),
					this.getPCPosition().getY() - 1);
			break;
		case KeyEvent.VK_DOWN:
			this.movePc(this.getPCPosition().getX(),
					this.getPCPosition().getY() + 1);
			break;
		case KeyEvent.VK_LEFT:
			this.movePc(this.getPCPosition().getX() - 1, this.getPCPosition()
					.getY());
			break;
		case KeyEvent.VK_RIGHT:
			this.movePc(this.getPCPosition().getX() + 1, this.getPCPosition()
					.getY());
			break;
			
		case KeyEvent.VK_S:
			
			int selectedItemIndex = 0;
			Skill selectedSkill = null;
			
			while (true) {
				
				Gui.clearScreen();
				Gui.getConsole().printTextLn("Skills:");
				
				for (int i = 0; i < SkillType.values().length; i++) {
					
					Skill skill = this.getPc().getSkills().getSkill(SkillType.values()[i]);
					
					Gui.getConsole().printTextLn(skill.getType().getName() + " " + skill.getCombatValue() + "/" + skill.getMaxValue());
					
					if (i == selectedItemIndex) {
						selectedSkill = skill;
						Gui.getConsole().setScreenColorXY(0, i+1, Gui.getConsole().getScreenResolutionX(), i+2, Color.YELLOW);
					}
					
				}
				
				Gui.getConsole().printTextLn("");
				Gui.getConsole().printTextLn("Skill points left: "  + this.getPc().getSkills().getSkillPoints());
				
				Gui.getConsole().printTextLn("[UP][DOWN] to select");
				Gui.getConsole().setScreenColorXY(1, Gui.getConsole().getCursorY()-2, 3, Gui.getConsole().getCursorY()-1, Color.GREEN);
				Gui.getConsole().setScreenColorXY(5, Gui.getConsole().getCursorY()-2, 9, Gui.getConsole().getCursorY()-1, Color.GREEN);
				Gui.getConsole().printTextLn("[LEFT][RIGHT] to assign points");
				Gui.getConsole().setScreenColorXY(1, Gui.getConsole().getCursorY()-2, 5, Gui.getConsole().getCursorY()-1, Color.GREEN);
				Gui.getConsole().setScreenColorXY(7, Gui.getConsole().getCursorY()-2, 12, Gui.getConsole().getCursorY()-1, Color.GREEN);
				Gui.getConsole().printTextLn("[ESC] to leave");
				Gui.getConsole().setScreenColorXY(1, Gui.getConsole().getCursorY()-2, 4, Gui.getConsole().getCursorY()-1, Color.GREEN);
				
				KeyEvent skillKey = Gui.getNextKey();
				
				if (skillKey.getKeyCode() == KeyEvent.VK_UP) {
					
					SoundSystem.getInstance().bloop();
					
					if (selectedItemIndex > 0) {
						selectedItemIndex--;
					}
					
				} else if (skillKey.getKeyCode() == KeyEvent.VK_DOWN) {
					
					SoundSystem.getInstance().bleep();
					
					if (selectedItemIndex < SkillType.values().length - 1) {
						selectedItemIndex++;
					}
					
				} else if (skillKey.getKeyCode() == KeyEvent.VK_LEFT) {
					
					SoundSystem.getInstance().bloop();
					
					this.getPc().getSkills().uninvestTo(selectedSkill.getType());
					
				} else if (skillKey.getKeyCode() == KeyEvent.VK_RIGHT) {
					
					SoundSystem.getInstance().bleep();
					
					this.getPc().getSkills().investTo(selectedSkill.getType());
					
				} else if (skillKey.getKeyCode() == KeyEvent.VK_ESCAPE) {
					
					break;
					
				}
				
				
				
			}
			
			break;
			
		case KeyEvent.VK_A:
			
			int selectedAttIndex = 0;
			Attribute selectedAttribute = null;
			
			while (true) {
				
				Gui.clearScreen();
				Gui.getConsole().printTextLn("Attributes:");
				
				for (int i = 0; i < AttributeType.values().length; i++) {
					
					Attribute attribute = this.getPc().getAttributes().getAttribute(AttributeType.values()[i]);
					
					Gui.getConsole().printTextLn(attribute.getType().getName() + " " + attribute.getMinValue() + "/" + attribute.getCombatValue() + "/" + attribute.getMaxValue());
					
					if (i == selectedAttIndex) {
						selectedAttribute = attribute;
						Gui.getConsole().setScreenColorXY(0, i+1, Gui.getConsole().getScreenResolutionX(), i+2, Color.YELLOW);
					}
					
				}
				
				Gui.getConsole().printTextLn("");
				Gui.getConsole().printTextLn("Attribute points left: "  + this.getPc().getAttributes().getAttributePoints());
				
				Gui.getConsole().printTextLn("[UP][DOWN] to select");
				Gui.getConsole().setScreenColorXY(1, Gui.getConsole().getCursorY()-2, 3, Gui.getConsole().getCursorY()-1, Color.GREEN);
				Gui.getConsole().setScreenColorXY(5, Gui.getConsole().getCursorY()-2, 9, Gui.getConsole().getCursorY()-1, Color.GREEN);
				Gui.getConsole().printTextLn("[LEFT][RIGHT] to assign points");
				Gui.getConsole().setScreenColorXY(1, Gui.getConsole().getCursorY()-2, 5, Gui.getConsole().getCursorY()-1, Color.GREEN);
				Gui.getConsole().setScreenColorXY(7, Gui.getConsole().getCursorY()-2, 12, Gui.getConsole().getCursorY()-1, Color.GREEN);
				Gui.getConsole().printTextLn("[ESC] to leave");
				Gui.getConsole().setScreenColorXY(1, Gui.getConsole().getCursorY()-2, 4, Gui.getConsole().getCursorY()-1, Color.GREEN);
				
				KeyEvent skillKey = Gui.getNextKey();
				
				if (skillKey.getKeyCode() == KeyEvent.VK_UP) {
					
					SoundSystem.getInstance().bloop();
					
					if (selectedAttIndex > 0) {
						selectedAttIndex--;
					}
					
				} else if (skillKey.getKeyCode() == KeyEvent.VK_DOWN) {
					
					SoundSystem.getInstance().bleep();
					
					if (selectedAttIndex < SkillType.values().length - 1) {
						selectedAttIndex++;
					}
					
				} else if (skillKey.getKeyCode() == KeyEvent.VK_LEFT) {
					
					SoundSystem.getInstance().bloop();
					
					this.getPc().getAttributes().uninvestTo(selectedAttribute.getType());
					
				} else if (skillKey.getKeyCode() == KeyEvent.VK_RIGHT) {
					
					SoundSystem.getInstance().bleep();
					
					this.getPc().getAttributes().investTo(selectedAttribute.getType());
					
				} else if (skillKey.getKeyCode() == KeyEvent.VK_ESCAPE) {
					
					break;
					
				}
				
				
				
			}
			
			break;
			
		case KeyEvent.VK_P:
			// pickup item

			Cell tile = this.getArea().getTile()[this.getPCPosition().getX()][this
					.getPCPosition().getY()];

			if (!tile.getItems().isEmpty()) {
				Item tmp = tile.getItems().remove(0);
				this.getPc().getInventory().addItem(tmp);
				this.getMessageBuffer().addLast(
						"Picked up " + tmp.getName() + ".");
			} else {
				this.getMessageBuffer().addLast("Nothing to pick up.");
			}

			break;
			
		case KeyEvent.VK_B:
			
			while (true) {
			
				Gui.clearScreen();
				Gui.getConsole().printTextLn("Skillbook:");
				
				for (Ability ability : this.getPc().getAbilities()) {
					Gui.getConsole().printTextLn(ability.getName() + " " + ability.getCost().getMpCost() + "MP");
				}
				
				List<MenuItem> abilityMenu = new ArrayList<MenuItem>();
				
				MenuItem learn = new MenuItem(null, 'A', "Learn a ability");
				abilityMenu.add(learn);
				MenuItem ready = new MenuItem(null, 'R', "Ready ability");
				abilityMenu.add(ready);
				MenuItem leaveSkillbook = new MenuItem(null, 'L', "Leave");
				abilityMenu.add(leaveSkillbook);
	
				MenuItem selectedAbilityMenuItem = Gui.getInlineMenuItem(abilityMenu, leaveSkillbook, Gui.getConsole().getCursorX(), Gui.getConsole().getCursorY());
				
				if (selectedAbilityMenuItem == leaveSkillbook) {
					break;
				} else if (selectedAbilityMenuItem == learn) {
					
					List<MenuItem> menuModel = new ArrayList<MenuItem>();
					for (int i = 0; i < this.getPc().getInventory().getBackpack().size(); i++) {
						menuModel.add(new MenuItem(this.getPc().getInventory().getBackpack().get(i).getGraphics(), (char)0, this.getPc().getInventory().getBackpack().get(i).getName()));
					}
					
					Integer index = Gui.getMenuItemIndex(menuModel, "Select item to grant skill");
					
					if (index != null) {
						Item item = this.getPc().getInventory().getBackpack().get(index);
						if (item != null
								&& item.getItemRoles().contains(
										ItemRoles.ABILITY_CARRIER)) {
							if (item.getLinkedAbility() != null && !this.getPc().getAbilities().contains(item.getLinkedAbility())) { 
								this.getPc().getAbilities().add(item.getLinkedAbility());
								this.getPc().getInventory().getBackpack().remove(item);
								this.getPc().setMP(this.getPc().getMP() + 10);
							}
						}
					}
					
				} else if (selectedAbilityMenuItem == ready) {
					
					List<MenuItem> menuModel = new ArrayList<MenuItem>();
					for (int i = 0; i < this.getPc().getAbilities().size(); i++) {
						menuModel.add(new MenuItem(null, (char)0, this.getPc().getAbilities().get(i).getName()));
					}
					
					Integer index = Gui.getMenuItemIndex(menuModel, "Select skill to ready");
					
					if (index != null) {
						
						this.readyAbility = this.getPc().getAbilities().get(index);
						
					}
					
				}
			
			}
			
			break;
			
		case KeyEvent.VK_L:
			// look around and target
			
			Cell targetCell = Gui.targetTile(this, Target.ACTOR);
			
			if (targetCell != null) {
				this.setSelectedTarget(targetCell.getActor());
			}
			
			break;
			
		case KeyEvent.VK_TAB:
			// Tab targets closest monster and cycles between available targets.
			
			int[][] los = AreaTools.checkLOS(this.getPc().getCurrentArea().getActorTile(this.getPc()).getPosition(),
					this.getPc().getCurrentArea(), this.getPc().getLOSDistance());

			List<Position> visibleNPCs = new ArrayList<Position>();
			
			for (int iX = 0; iX < los.length; iX++) {
				for (int iY = 0; iY < los[iX].length; iY++) {
					if (los[iX][iY] == 1) {
						if (this.getPc().getCurrentArea().getTile()[iX][iY].getActor() != null) {
							visibleNPCs.add(new Position(iX, iY));
						}
					}
				}
			}
			
			if (visibleNPCs.size() > 0) {
			
				List<Position> sorted = AreaTools.sortByDistance(visibleNPCs, this.getPCPosition());
				Position chosenPosition = null;
				
				if (this.getSelectedTarget() == null ) {
					
					chosenPosition = sorted.get(0);
					
				} else {
					
					Position currentTargetPosition = this.getSelectedTarget().getCurrentArea().getActorTile(this.getSelectedTarget()).getPosition();
					
					for (int i = 0; i < sorted.size(); i++) {
						if (sorted.get(i).equals(currentTargetPosition)) {
							
							if (i+1 >= sorted.size()) {
								chosenPosition = sorted.get(0);
							} else {
								chosenPosition = sorted.get(i+1);
							}
							
							break;
							
						}
					}
					
					if (chosenPosition == null) {
						chosenPosition = sorted.get(0);
						// original target did not match - disappeared between cycling or moved, so we start cycle anew
					}
					
				}
			
				this.setSelectedTarget(this.getArea().getTile()[chosenPosition.getX()][chosenPosition.getY()].getActor());
			} else {
				this.setSelectedTarget(null);
			}
			
			break;
			
		case KeyEvent.VK_U:
			
			if (this.readyAbility != null) {
				
				AbilityCost ablityCost = this.readyAbility.getCost();
				
				if (this.getPc().validateCost(ablityCost)) {
					
					
					// target aquisition
					
					Actor target = this.getSelectedTarget();
					MagicalProperty abilityRaw = this.readyAbility.getMagicalProperty(this.getPc().getSkills().getSkill(this.readyAbility.getLinkedSkill()).getCombatValue());
					
					if (this.readyAbility.getTarget() != Target.TILE) {
						
						// actor target
						
						if (target == null) {
							if (this.readyAbility.getTarget() == Target.ACTOR) {
								Cell targetTile = Gui.targetTile(this, this.readyAbility.getTarget());
								if (targetTile != null) {
									target = targetTile.getActor();
									this.setSelectedTarget(target);
								} else {
									return;
								}
							} else if (this.readyAbility.getTarget() == Target.SELF) {
								target = this.getPc();
							}
						}
						
						this.getPc().processCostPacket(ablityCost);
						abilityRaw.onCast(this.getPc(), target);

					} else {
						
						// tile target
						Cell targetTile = Gui.targetTile(this, this.readyAbility.getTarget());
						if (targetTile != null) {
							// set actor if present
							target = targetTile.getActor();
							this.setSelectedTarget(target);
						} else {
							return;
						}
						
						this.getPc().processCostPacket(ablityCost);
						abilityRaw.onCast(this.getPc(), targetTile);
						
					}
					
					this.getMessageBuffer().addLast("You cast " + this.readyAbility.getName());
					
					// award xp for using ability
					XPPacket xp = new XPPacket();
					xp.getSkillXP().put(this.readyAbility.getLinkedSkill(), 10);
					this.getPc().processXPPacket(xp );
					
				}
			}
			
			break;
			
		case KeyEvent.VK_O:
			
			Gui.clearScreen();
			Gui.getConsole().printTextLn("Character: " + this.getPc().getName());
			Gui.getConsole().printTextLn("");
			Gui.getConsole().printTextLn("Armor: " + this.getPc().getAR());
			Gui.getConsole().printTextLn("Block: " + this.getPc().getBlock()+"%");
			Gui.getConsole().printTextLn("APs: " + this.getPc().getTotalActionPoints());
			Gui.getConsole().printTextLn("");
			Gui.getConsole().printTextLn("Unused Skl P: " + this.getPc().getSkills().getSkillPoints());
			Gui.getConsole().printTextLn("Unused Atr P: " + this.getPc().getAttributes().getAttributePoints());
			Gui.getConsole().printTextLn("");
			Gui.getConsole().printTextLn("Effects:");
			
			for (Effect effect : this.getPc().getEffectStack().getEffects()) {
				Gui.getConsole().printTextLn(effect.getName()+" "+effect.getTimer().getDuration()+"/"+effect.getTimer().getExpiration());
			}
			
			Gui.getNextKey();
			
			break;
			
		case KeyEvent.VK_T:
			
			if (this.getSelectedTarget() != null && this.getSelectedTarget() != this.getPc()) {
				
				Conversation dialogue = this.getSelectedTarget().getAI().getDialogue();
				
				if (dialogue != null) {
					
					// wee, we are talkin.
					
					State actualState = dialogue.getStates().get(dialogue.getInital());
					
					while (true) {
						
						Gui.getConsole().cls();
						Gui.getConsole().moveCursorTo(1, 1);
						
						Gui.getConsole().printTextLn(actualState.getText());
						
						List<MenuItem> menuModel = new LinkedList<MenuItem>();
						
						int i = 0;
						for (Transition transition : actualState.getTransitions()) {
							menuModel.add(new MenuItem(null,(char)((int)'a'+i),transition.getText()));
							i++;
						}
						
						MenuItem item = Gui.getInlineMenuItem(menuModel, null, Gui.getConsole().getCursorX(), Gui.getConsole().getCursorY());
						String to = actualState.getTransitions().get(menuModel.indexOf(item)).getTo();
						
						if (dialogue.getFin().equals(to)) {
							break;
						} else {
							actualState = dialogue.getStates().get(to);
						}
						
					}
					
				}
				
			}
			
			break;

		case KeyEvent.VK_X:

			while (true) {
				
				Gui.clearScreen();
				Gui.getConsole().printTextLn(
						"Experience: " + this.getPc().getUnclaimedXP());

				for (AttributeType attibuteType : AttributeType.values()) {
					Gui.getConsole().printTextLn(
									"("
							+(this.getPc().getAttributes().getAttributes()
									.get(attibuteType).isMarkedLo() ? "*":" " )
									+ "/"
							+(this.getPc().getAttributes().getAttributes()
									.get(attibuteType).isMarkedHi() ? "*":" " )
									+ ")"
									+ attibuteType.getName()
									+ ": "
									+ this.getPc().getAttributes().getAttributes()
											.get(attibuteType).getXpHi()
									+ "/"
									+ this.getPc().getAttributes().getAttributes()
											.get(attibuteType).getXpLo());
				}

				for (SkillType skillType : SkillType.values()) {
					Gui.getConsole().printTextLn(
							(this.getPc().getSkills().getSkill(skillType).isMarked()? "(*)":"( )" )
									+ skillType.getName()
									+ ": "
									+ this.getPc().getSkills().getSkill(skillType)
											.getXp());
				}
				
				Gui.getConsole().printTextLn("");
				
				List<MenuItem> itemMenu = new ArrayList<MenuItem>();
				
				MenuItem mark = new MenuItem(null, 'M', "Mark");
				itemMenu.add(mark);
				MenuItem unmark = new MenuItem(null, 'U', "Unmark");
				itemMenu.add(unmark);
				MenuItem leave = new MenuItem(null, 'L', "Leave");
				itemMenu.add(leave);

				MenuItem selectedMenuItem = Gui.getInlineMenuItem(itemMenu, leave, Gui.getConsole().getCursorX(), Gui.getConsole().getCursorY());
				
				if (selectedMenuItem == leave) {
					break;
				} else if (selectedMenuItem == mark) {
					
					Gui.getConsole().printTextLn("");
					
					List<MenuItem> typeMenu = new ArrayList<MenuItem>();
					
					MenuItem skill = new MenuItem(null, 'S', "Skill");
					typeMenu.add(skill);
					MenuItem hiAtt = new MenuItem(null, 'H', "High Attribute");
					typeMenu.add(hiAtt);
					MenuItem loAtt = new MenuItem(null, 'L', "Low Attribute");
					typeMenu.add(loAtt);

					MenuItem selectedTypeMenuItem = Gui.getInlineMenuItem(typeMenu, null, Gui.getConsole().getCursorX(), Gui.getConsole().getCursorY());
					
					if (selectedTypeMenuItem == skill) {
						
						List<MenuItem> maskSkillMenuModel = new ArrayList<MenuItem>();
						for (int i = 0; i < SkillType.values().length; i++) {
							maskSkillMenuModel.add(new MenuItem(null, (char)0, SkillType.values()[i].getName()));
						}
						
						Integer markSkillIndex = Gui.getMenuItemIndex(maskSkillMenuModel, "Select skill to mark");
						
						if (markSkillIndex != null) {
							this.getPc().getSkills().getSkill(SkillType.values()[markSkillIndex]).setMarked(true);
						}

					} else if (selectedTypeMenuItem == hiAtt) {
						
						List<MenuItem> markHiAttMenuModel = new ArrayList<MenuItem>();
						for (int i = 0; i < AttributeType.values().length; i++) {
							markHiAttMenuModel.add(new MenuItem(null, (char)0, AttributeType.values()[i].getName()));
						}
						
						Integer markHiAttIndex = Gui.getMenuItemIndex(markHiAttMenuModel, "Select attribute to mark");
						
						if (markHiAttIndex != null) {
							this.getPc().getAttributes().getAttributes().get(AttributeType.values()[markHiAttIndex]).setMarkedHi(true);
						}
						

					} else if (selectedTypeMenuItem == loAtt) {
						
						List<MenuItem> markLoAttMenuModel = new ArrayList<MenuItem>();
						for (int i = 0; i < AttributeType.values().length; i++) {
							markLoAttMenuModel.add(new MenuItem(null, (char)0, AttributeType.values()[i].getName()));
						}
						
						Integer markLoAttIndex = Gui.getMenuItemIndex(markLoAttMenuModel, "Select attribute to mark");
						
						if (markLoAttIndex != null) {
							this.getPc().getAttributes().getAttributes().get(AttributeType.values()[markLoAttIndex]).setMarkedLo(true);
						}
						
					}
					
				} else if (selectedMenuItem == unmark) {
					
					Gui.getConsole().printTextLn("");
					
					List<MenuItem> typeMenu = new ArrayList<MenuItem>();
					
					MenuItem skill = new MenuItem(null, 'S', "Skill");
					typeMenu.add(skill);
					MenuItem hiAtt = new MenuItem(null, 'H', "High Attribute");
					typeMenu.add(hiAtt);
					MenuItem loAtt = new MenuItem(null, 'L', "Low Attribute");
					typeMenu.add(loAtt);

					MenuItem selectedTypeMenuItem = Gui.getInlineMenuItem(typeMenu, null, Gui.getConsole().getCursorX(), Gui.getConsole().getCursorY());
					
					if (selectedTypeMenuItem == skill) {
						
						List<MenuItem> maskSkillMenuModel = new ArrayList<MenuItem>();
						for (int i = 0; i < SkillType.values().length; i++) {
							maskSkillMenuModel.add(new MenuItem(null, (char)0, SkillType.values()[i].getName()));
						}
						
						Integer markSkillIndex = Gui.getMenuItemIndex(maskSkillMenuModel, "Select skill to unmark");
						
						if (markSkillIndex != null) {
							this.getPc().getSkills().getSkill(SkillType.values()[markSkillIndex]).setMarked(false);
						}

					} else if (selectedTypeMenuItem == hiAtt) {
						
						List<MenuItem> unMarkHiAttMenuModel = new ArrayList<MenuItem>();
						for (int i = 0; i < AttributeType.values().length; i++) {
							unMarkHiAttMenuModel.add(new MenuItem(null, (char)0, AttributeType.values()[i].getName()));
						}
						
						Integer unMarkHiAttIndex = Gui.getMenuItemIndex(unMarkHiAttMenuModel, "Select attribute to unmark");
						
						if (unMarkHiAttIndex != null) {
							this.getPc().getAttributes().getAttributes().get(AttributeType.values()[unMarkHiAttIndex]).setMarkedHi(false);
						}

					} else if (selectedTypeMenuItem == loAtt) {
						
						List<MenuItem> unMarkLoAttMenuModel = new ArrayList<MenuItem>();
						for (int i = 0; i < AttributeType.values().length; i++) {
							unMarkLoAttMenuModel.add(new MenuItem(null, (char)0, AttributeType.values()[i].getName()));
						}
						
						Integer UnMarkLoAttIndex = Gui.getMenuItemIndex(unMarkLoAttMenuModel, "Select attribute to unmark");
						
						if (UnMarkLoAttIndex != null) {
							this.getPc().getAttributes().getAttributes().get(AttributeType.values()[UnMarkLoAttIndex]).setMarkedLo(false);
						}
						
					}
					
				}
			}

			break;

		case KeyEvent.VK_I:
			// inventory

			while (true) {
				
				Gui.clearScreen();
				Gui.getConsole().printTextLn("Equipment:");
				
				Collection<Slot> slots = this.getPc().getInventory().getSlots();
				List<Item> inventory = this.getPc().getInventory().getBackpack();			
				
				for (Iterator<Slot> it = slots.iterator(); it.hasNext();) {
					Slot slot = it.next();
					
					Gui.getConsole().printText(slot.getSlotName()+" : ");
					
					if (slot.getSlotContent() != null) {
						Gui.getConsole().printTextLn(slot.getSlotContent().getName());
					} else {
						Gui.getConsole().printTextLn("empty");
					}

				}
				
				Gui.getConsole().printTextLn("");
				Gui.getConsole().printTextLn("Backpack:");
				
				for (int i = 0; i < (inventory.size() < 10 ? inventory.size() : 10 ); i++) {
					
					Item item = inventory.get(i);
					
					Gui.getConsole().printTextLn(item.getName());
					
				}

				Gui.getConsole().printTextLn("");
				
				List<MenuItem> itemMenu = new ArrayList<MenuItem>();
				
				MenuItem equip = new MenuItem(null, 'E', "Equip");
				itemMenu.add(equip);
				MenuItem unequip = new MenuItem(null, 'U', "Unequip");
				itemMenu.add(unequip);
				MenuItem drop = new MenuItem(null, 'D', "Drop");
				itemMenu.add(drop);
				MenuItem upgrade = new MenuItem(null, 'G', "Upgrade");
				itemMenu.add(upgrade);
				MenuItem examine = new MenuItem(null, 'X', "Examine");
				itemMenu.add(examine);
				MenuItem consume = new MenuItem(null, 'C', "Consume");
				itemMenu.add(consume);
				MenuItem leave = new MenuItem(null, 'L', "Leave");
				itemMenu.add(leave);

				MenuItem selectedMenuItem = Gui.getInlineMenuItem(itemMenu, leave, Gui.getConsole().getCursorX(), Gui.getConsole().getCursorY());
				
				if (selectedMenuItem == consume) {
					
					List<MenuItem> menuModel = new ArrayList<MenuItem>();
					for (int i = 0; i < this.getPc().getInventory().getBackpack().size(); i++) {
						menuModel.add(new MenuItem(this.getPc().getInventory().getBackpack().get(i).getGraphics(), (char)0, this.getPc().getInventory().getBackpack().get(i).getName()));
					}
					
					Integer index = Gui.getMenuItemIndex(menuModel, "Select item to consume");
					
					if (index != null) {
						Item item = this.getPc().getInventory().getBackpack().get(index);
						if (item != null
								&& item.getItemRoles().contains(
										ItemRoles.CONSUMBALE)) {
							item.consume(this.getPc());
							this.getPc().getInventory().getBackpack().remove(item);
						}
					}

				} else if (selectedMenuItem == equip) {
					
					List<MenuItem> slotsEquipMenuModel = new ArrayList<MenuItem>();
					for (int i = 0; i < this.getPc().getInventory().getSlots().size(); i++) {
						slotsEquipMenuModel.add(new MenuItem(null, (char)0, this.getPc().getInventory().getSlots().get(i).getSlotName() 
								+ " : " + (this.getPc().getInventory().getSlots().get(i).getSlotContent()!= null ? this.getPc().getInventory().getSlots().get(i).getSlotContent().getName() : "empty" )));
					}
					
					Integer to = Gui.getMenuItemIndex(slotsEquipMenuModel, "Select equipment slot");
					
					if (to != null) {
					
						List<MenuItem> equipMenuModel = new ArrayList<MenuItem>();
						for (int i = 0; i < this.getPc().getInventory().getBackpack().size(); i++) {
							equipMenuModel.add(new MenuItem(this.getPc().getInventory().getBackpack().get(i).getGraphics(), (char)0, this.getPc().getInventory().getBackpack().get(i).getName()));
						}
						
						Integer from = Gui.getMenuItemIndex(equipMenuModel, "Select item to equip");
						
						if (from != null) {
							this.getPc().getInventory().equipItem(from, to);
						}
					
					}

				} else if (selectedMenuItem == unequip) {

					List<MenuItem> slotsUnequipMenuModel = new ArrayList<MenuItem>();
					for (int i = 0; i < this.getPc().getInventory().getSlots().size(); i++) {
						slotsUnequipMenuModel.add(new MenuItem(null, (char)0, this.getPc().getInventory().getSlots().get(i).getSlotName() 
								+ " : " + (this.getPc().getInventory().getSlots().get(i).getSlotContent()!= null ? this.getPc().getInventory().getSlots().get(i).getSlotContent().getName() : "empty" )));
					}

					Integer unequipFrom = Gui.getMenuItemIndex(slotsUnequipMenuModel, "Select equipment slot");
					
					if (unequipFrom != null) {
						this.getPc().getInventory().unEquipItem(unequipFrom);
					}

				} else if (selectedMenuItem == drop) {

					List<MenuItem> dropMenuModel = new ArrayList<MenuItem>();
					for (int i = 0; i < this.getPc().getInventory().getBackpack().size(); i++) {
						dropMenuModel.add(new MenuItem(this.getPc().getInventory().getBackpack().get(i).getGraphics(), (char)0, this.getPc().getInventory().getBackpack().get(i).getName()));
					}

					Integer dropFrom = Gui.getMenuItemIndex(dropMenuModel, "Select item to drop");
					
					if (dropFrom != null) {
						Item droppedItem = this.getPc().getInventory().getBackpack().get(dropFrom);
						this.getArea().getTile()[this.getPCPosition().getX()][this.getPCPosition().getY()].getItems().add(droppedItem);
						this.getPc().getInventory().getBackpack().remove(droppedItem);
					}

				} else if (selectedMenuItem == leave) {
					
					SoundSystem.getInstance().cancel();
					
					break ;
				} else if (selectedMenuItem == upgrade) {
					
					List<MenuItem> menuModel = new ArrayList<MenuItem>();
					for (int i = 0; i < this.getPc().getInventory().getBackpack().size(); i++) {
						menuModel.add(new MenuItem(this.getPc().getInventory().getBackpack().get(i).getGraphics(), (char)0, this.getPc().getInventory().getBackpack().get(i).getName()));
					}
					
					Integer index = Gui.getMenuItemIndex(menuModel, "Select item to upgrade");
					
					if (index != null) {
						Item item = this.getPc().getInventory().getBackpack().get(index);
						if (item != null && item.getSockets().size() > 0) {
							
							List<MenuItem> itemSocketsModel = new ArrayList<MenuItem>();
							for (int i = 0; i < item.getSockets().size(); i++) {
								itemSocketsModel.add(new MenuItem(null, (char)0, item.getSockets().get(i).getRoles() + " : " + ((item.getSockets().get(i).getContent() == null) ? "empty" : item.getSockets().get(i).getContent().getName())));
							}
							
							Integer slotIndex = Gui.getMenuItemIndex(itemSocketsModel, "Select upgrade slot");
							
							if (slotIndex != null) {
								ItemSocket slot = item.getSockets().get(slotIndex);
								
								Integer upgradeIndex = Gui.getMenuItemIndex(menuModel, "Select upgrade");
								
								if (upgradeIndex != null) {
									
									Item upgradeItem = this.getPc().getInventory().getBackpack().get(upgradeIndex);
									
									outer: for (ItemRoles itemRoles : upgradeItem.getItemRoles()) {
										for (ItemRoles slotRoles : slot.getRoles()) {
											if (itemRoles == slotRoles) {
												slot.setContent(upgradeItem);
												this.getPc().getInventory().getBackpack().remove(upgradeItem);
												break outer;
											}
										}
									}
									
									
									
								}
								
							}
							
						}
					}
					
				} else if (selectedMenuItem == examine) {
					
					List<MenuItem> menuModel = new ArrayList<MenuItem>();
					for (int i = 0; i < this.getPc().getInventory().getBackpack().size(); i++) {
						menuModel.add(new MenuItem(this.getPc().getInventory().getBackpack().get(i).getGraphics(), (char)0, this.getPc().getInventory().getBackpack().get(i).getName()));
					}
					
					Integer index = Gui.getMenuItemIndex(menuModel, "Select item to examine");
					
					if (index != null) {
						Item item = this.getPc().getInventory().getBackpack().get(index);
						
						Gui.clearScreen();
						Gui.getConsole().printTextLn(item.getName());
						Gui.getConsole().printTextLn("");
						Gui.getConsole().printTextLn(item.getBlessing().toString());
						if (item.getMinDamage() != 0 && item.getMaxDamage() != 0 ) {
							Gui.getConsole().printTextLn("Damage: " + item.getMinDamage() + " - " + item.getMaxDamage());
						}
						if (item.getDefence() != 0) {
							Gui.getConsole().printTextLn("Armor: " + item.getDefence());
						}
						if (item.getBlock() != 0) {
							Gui.getConsole().printTextLn("Block: " + item.getBlock());
						}
						Gui.getConsole().printTextLn("");
						Gui.getConsole().printTextLn("Magic:");
						for (MagicalProperty magic : item.getMagicalProperties()) {
							if (magic.isPrefix()) {
								Gui.getConsole().printTextLn(magic.getPrefix());
							} else if (magic.isSuffix()) {
								Gui.getConsole().printTextLn(magic.getSuffix());
							}
						}
						Gui.getConsole().printTextLn("");
						Gui.getConsole().printTextLn("Upgrades:");
						for (ItemSocket socket : item.getSockets()) {
							Gui.getConsole().printTextLn(socket.getRoles() + " : " + ((socket.getContent() == null) ? "empty" : socket.getContent().getName()));
						}
						
						Gui.getNextKey();
						
					}
					
				}

			}
		default:
			break;
		}

	}

	/**
	 * @return Returns the pc.
	 */
	public Actor getPc() {
		return this.pc;
	}

	/**
	 * @param pc
	 *            The pc to set.
	 */
	public void setPc(Actor pc) {
		this.pc = pc;
		this.getPc().setCurrentModel(this);
	}

	public void setArea(Area area) {
		this.area = area;
		this.getPc().setCurrentArea(area);
	}

	/**
	 * @return Returns the area.
	 */
	public Area getArea() {
		return this.area;
	}

	/**
	 * @return Returns the position.
	 */
	public Position getPCPosition() {
		return this.position;
	}

	/**
	 * @param position
	 *            The position to set.
	 */
	public void setPosition(Position position) {
		this.position = position;
	}

	public Representation[][] buildScreen(int width, int height) {

		Representation[][] result = new Representation[width][height];
		// Two top lines, one bottom line, one line at left and are reserved

		// 1st, create clear screen
		for (int i = 0; i < result.length; i++) {
			for (int j = 0; j < result[i].length; j++) {
				// flyweight is not enough for this.
				result[i][j] = Representation.getEmpty().clone();
			}
		}

		int centerX = (width - 1) / 2;
		int centerY = (height - 1) / 2;

		for (int x = 1; x < width; x++) {
			for (int y = 1; y < height - 2; y++) {

				int areaX = x - (centerX - this.getPCPosition().getX());
				int areaY = y - (centerY - this.getPCPosition().getY());

				if (!AreaTools.checkAreaBounds(areaX, areaY, area)) {
					result[x][y] = Representation.getEmpty().clone();
				} else {

					Cell tile = this.getArea().getTile()[areaX][areaY];

					if (tile.isKnown()) {

						if (tile.isVisible()) {

							if (tile.getActor() != null) {
								result[x][y] = tile.getActor().getGraphics().clone();
							} else if (tile.getEffectStack().getEffects().size() != 0) {
								result[x][y] = tile.getEffectStack().getEffects().get(0).getGraphics().clone();
							} else if (tile.getItems().size() != 0) {
								result[x][y] = tile.getItems().get(0)
										.getGraphics().clone();
							} else {
								result[x][y] = tile.getGraphics().clone();
							}
							
						} else {
							
							if (tile.getItems().size() != 0) {
								result[x][y] = tile.getItems().get(0)
										.getGraphics().clone();
							} else {
								result[x][y] = tile.getGraphics().clone();
							}
							
							result[x][y].setModelColor(result[x][y].getModelColor().darker().darker());
							
						}
					}
				}

			}
		}

		result[centerX][centerY] = this.pc.getGraphics().clone();

		return result;

	}

	public void movePc(int x, int y) {
		if (AreaTools.checkAreaBounds(x, y, this.getArea())) {
			
			Cell targetTile = this.getArea().getTile()[x][y];
			
			if (this.getPc().getActionPoints() > 0 ) {
			
			if (targetTile.isMoveThrough() == true) {
				if (targetTile.getActor() == null || targetTile.getActor() == this.getPc()) {
					// ok, we will move NOW
					
					this.getPc().setActionPoints(this.getPc().getActionPoints() - 1 ); 

					this.getArea().getTile()[this.getPCPosition().getX()][this
							.getPCPosition().getY()].setActor(null);
					targetTile.setActor(this.getPc());
					this.setPosition(new Position(x, y));

					this.getArea().setLOSmap(
							AreaTools.checkLOS(this.getPCPosition(), this
									.getArea(), this.getPc().getLOSDistance()));
					
					if (targetTile.getPortal() != null && targetTile.getPortal().getRemotePortal() != null) {
						// pc just moved to portal, lets activate it.
						
						// 1) 
						
						Area targetArea = this.getArea();
						
						if (targetTile.getPortal().getRemoteArea() != null) {
							targetArea = AreaFactory.getInstance().getArea(this, targetTile.getPortal().getRemoteArea());
						}
						
						// now, lets get destination tile
						
						Cell targetPortalTile = targetArea.getTileByPortalKey(targetTile.getPortal().getRemotePortal());
						
						targetPortalTile.setActor(targetTile.getActor());
						targetTile.setActor(null);
						
						this.setArea(targetArea);
						this.pcEnterArea(targetPortalTile.getPosition().getX(), targetPortalTile.getPosition().getY());
						
					}
					
				} else {
					// oy, we found moster, we try to kill it.
					
					this.getPc().setActionPoints(this.getPc().getActionPoints() -1 );

					Actor monster = this.getArea().getTile()[x][y].getActor();

					this.setSelectedTarget(monster);
					
					if (this.getPc().getFaction().getFoes().contains(monster.getFaction())) {
						this.getPc().melee(monster);
					} else {
						this.messageBuffer.addLast(monster.getName()
								+ " Blocks your way");
					}
				}
			} else {
				this.messageBuffer.addLast(this.getArea().getTile()[x][y]
						.getDescription()
						+ " Blocks your way");
				this.messageBuffer.addLast("You dont like it");
			}
			
			} else {
				//no can do, not enough action points.
			}
			
		} else {
			// no can do, out of range ...
		}

	}

	/**
	 * @return Returns the messageBuffer.
	 */
	public LinkedList<String> getMessageBuffer() {
		return this.messageBuffer;
	}

	public Ability getReadyAbility() {
		return this.readyAbility;
	}

	public void setReadyAbility(Ability readyAbility) {
		this.readyAbility = readyAbility;
	}

	public void fireMonsterEvents() {
		
		Area activeArea = this.getArea();

		synchronized (activeArea) {
			for (int x = 0; x < activeArea.getTile().length; x++) {
				for (int y = 0; y < activeArea.getTile()[x].length; y++) {
					Cell tile = activeArea.getTile()[x][y];
					tile.tick();
					if (tile.getActor() != null && tile.getActor() != this.getPc()) {
						tile.getActor().getAI().setActed(false);
						tile.getActor().tick();
					}
				}
			}
			
			for (int x = 0; x < activeArea.getTile().length; x++) {
				for (int y = 0; y < activeArea.getTile()[x].length; y++) {
					Cell tile = activeArea.getTile()[x][y];
					if (tile.getActor() != null && tile.getActor() != this.getPc() && !tile.getActor().getAI().getActed()) {
						Actor actor = tile.getActor();
						actor.getAI().setActed(true);
						actor.getAI().act();
					}
				}
			}
		}

	}

	public void moveActor(int sourceX, int sourceY, int targetX, int targetY) {

		if (sourceX == targetX && sourceY == targetY) {
			return;
		}

		if (AreaTools.checkAreaBounds(targetX, targetY, this.getArea())
				&& AreaTools.checkAreaBounds(sourceX, sourceY, this.getArea())) {
			if (this.getArea().getTile()[targetX][targetY].getActor() == null) {
				Actor movedActor = this.getArea().getTile()[sourceX][sourceY]
						.getActor();

				this.getArea().getTile()[sourceX][sourceY].setActor(null);
				this.getArea().getTile()[targetX][targetY].setActor(movedActor);
			} else {
				// tile is occupied
			}
		} else {
			// out of bounds, no movement
		}

	}

	public boolean inCombat() {
		
		for (int x = 0; x < this.getArea().getTile().length; x++) {
			for (int y = 0; y < this.getArea().getTile()[x].length; y++) {
				if (this.getArea().getTile()[x][y].isVisible() && this.getArea().getTile()[x][y].getActor() != null 
						&& this.getPc().getFaction().getFoes().contains(this.getArea().getTile()[x][y].getActor().getFaction())) {
					return true;
				}
			}
		}
		
		return false;
	}

	public void pcEnterArea(int x, int y) {
		this.setSelectedTarget(null);
		this.setPosition(new Position(x, y));
		this.getPc().setActionPoints(this.getPc().getActionPoints() + 1 ); // make sure teleport move wil not cost ap.
		this.movePc(x, y);
	}

	public void fireEnviromentEvents() {
		this.getPc().tick();
	}

	public void setSelectedTarget(Actor selectedTarget) {
		this.selectedTarget = selectedTarget;
	}

	public Actor getSelectedTarget() {
		return selectedTarget;
	}

}
